-- Advertising
advertising = {
    maxShowCount = 20, --每天最多弹的次数
    maxLevel = 10,
    
    -- 广告初始化信息
    Delegates = {
        AppLovin={},
        Chartboost={
            android={appId="557f9f5b43150f0840feed00", signature="096458e8735b5bd635f8c6f20e4694e3943e5383"},
            ios={appId="557f9f490d60256bd4ea38b1", signature="8cbb6d84826e6e5f0a4613caf81e5dda5b4f3d01"},
        },
        Unity={
            android={gameId="46509"},
            ios={gameId="46508"},
        },
        Vungle={
            android={appId="557fa1ad970cd76f72000259"},
            ios={appId="557f9fedb2addf5a5a0000c8"},
        },
    },
    -- 广告位及参数: {index=唯一的广告索引（必填）, delegate=广告商名 or 广告位名, name=广告位名, type=类型, rewardMultiply=奖励的乘数 or 1}
    Advertisings = {
        AppLovin_image = {index=11, delegate="AppLovin", type="INTERSTITIAL"},
        AppLovin_video = {index=12, delegate="AppLovin", type="VIDEO"},
        Chartboost = {index=21, name="game_ad"},
        Unity = {index=31},
        Vungle = {index=41},
    },
	-- 各个广告位
	Placements = {
		-- 地图上的图标
		MapIcon={minLV=0,
			names={"AppLovin_video", "Unity", "Vungle"},
			rewards ={[1] = {type = "cash", count = 2},
			          [2] = {type = "heart", count = 1}}
		},
		
		-- 退出买心面板
        BuyLifesMent = {minLV=0,
            names={"AppLovin_video", "Unity", "Vungle"},
            rewards ={[1] = {type = "heart", count = 1}}
        },
        
		-- 退出某关
		Settlement={minLV=7,
			names={"AppLovin_image", "Chartboost"},
		}
	},
}
-- Test IDs from Zoo
-- advertising.Delegates.Chartboost.ios = {appId="54dc4498c909a6580c0fe5ea", signature="93691780edbda7c5a0919c8c2918edd8e24e0b07"}
-- advertising.Delegates.Unity.ios = {gameId="131625774"}

-- 按代理名称和类型取得广告
function advertising:getAdvertisingForDelegateName(name, type)
    for key, adv in pairs(self.Advertisings) do
        if adv.delegate == name and (not type and true or adv.type == type) then
            return adv
        end
    end
    return nil
end
-- 按索引值取得广告
function advertising:getAdvertisingForIndex(index)
	for _, adv in pairs(self.Advertisings) do
		if adv.index == index then
			return adv
		end
	end
	return nil
end
-- 取得广告位
function advertising:getPlacement(placeName)
	return self.Placements[placeName]
end
-- 取得所有包含指定广告名的广告位
function advertising:getPlacementWithAdvertisingName(advName)
	local places = {}
	for _, place in pairs(self.Placements) do
		local found = false
		for _, name in pairs(place.names) do
			if name == advName then
				found = true
				break
			end
		end
		if found then table.insert(places, place) end
	end
	return
end
-- 取得玩家玩过的最大关数
function advertising:getPlayerMaxLevel()
	return MainValue.dataManager.levelManager:getMaxLevel()
end

function advertising:fireEvent(name, value)
	MainValue.eventDispatcher:dispatchEvent(name, value)
end

function advertising:addShowCount()
	-- TODO:
end
-- 初始化广告系统
function advertising:init()
    local platform = DataConfig.os
    local ad = ad
	-- print("advertising.init()", platform, ad)
    for name, dele in pairs(self.Delegates) do
		dele._name = name
		-- print("advertising.init() initDelegate", name)
        ad.initDelegate(name, dele[platform] or {})
    end
    for name, adv in pairs(self.Advertisings) do
		adv._name = name
        if not adv.delegate then
			adv.delegate = name
		end
		-- print("advertising.init() addAdvertising", adv.index)
        ad.addAdvertising(adv)
    end
    -- 设置事件监听函数
    -- type:  PRELOADED/REWARD/STATE_CHANGED
    -- event: {type=同上, delegate=广告商名, advertising=广告index, rewardAmount=奖励数值, state=广告状态, currency=货币类型}
	-- print("advertising.init() setEventListener")
    ad.setEventListener(function(type, event)
        print("advertising.EventListener()", type, event)
        if type == "PRELOADED" then
			-- 预加载
			self:fireEvent("AD_PRELOADED", event)
        elseif type == "REWARD" then
	        -- 获得奖励，给玩家加钱
			local place = self:getPlacement(self._lastPlacement)
			local rewards = place and place.rewards
			if rewards then
				self:fireEvent("AD_GOT_REWARD", rewards)
			end
        elseif type == "STATE_CHANGED" then
			-- 广告状态改变
            if event.state == "START_VIDEO" then
	            -- 关闭声音
				MainValue.soundManager:pauseMusic()
            elseif event.state == "EXIT" then
	            -- 恢复声音、增加广告播放次数
				MainValue.soundManager:resumeMusic()
				self:addShowCount()
            end
			self:fireEvent("AD_STATE_CHANGED", event)
        end
    end)
	-- print("advertising.init() done")
end

-- 获取指定广告位中可播的广告：玩家关数已达到、已载入
function advertising:getPlayableAdvertisingWithPlacement(placeName)
	local place = self:getPlacement(placeName)
	if place and (place.minLV or 0) < self:getPlayerMaxLevel() then
		for _, advName in ipairs(place.names) do
			local adv = self.Advertisings[advName]
			if adv and ad.isPreloaded(adv.index) then
				return adv
			end
		end
	end
	return nil
end

-- 显示指定广告位的广告。
-- 仅当玩家的关数超过广告位设定、能够按顺序取得已经缓存完成的广告时才会播放。
function advertising:showForPlacement(placeName)
	local place = self:getPlacement(placeName)
	if not place or (place.minLV or 0) > self:getPlayerMaxLevel() then
		return "NOT_AVAILIABLE_FOR_LEVEL"
	end
	local adv = self:getPlayableAdvertisingWithPlacement(placeName)
	local state = adv and ad.show(adv.index) or "NOT_AVAILIABLE"
	self._lastPlacement = state == "OK" and placeName or nil
	return state
end
