
local DBManager = class("DBManager");

local function setData(pkey, pdata)
    cc.UserDefault:getInstance():setStringForKey(pkey, json.encode(pdata));
end

local function flushData()
    cc.UserDefault:getInstance():flush();
end

local function getData(pkey)
    local temp = cc.UserDefault:getInstance():getStringForKey(pkey);
    return temp;
end

local function getIntegerForKey(pkey)
    local temp = cc.UserDefault:getInstance():getIntegerForKey(pkey);
    return temp;
end

local function setIntegerForKey(pkey, pdata)
    cc.UserDefault:getInstance():setIntegerForKey(pkey, pdata);
end

local function getLastId()

    local last_id = getIntegerForKey("last_id");

    if last_id == 0
    then
        last_id = 1;
        setIntegerForKey("last_id",last_id);
        flushData();
    end

    return last_id;
end

DBManager.uid = nil;
--需要本地存储
DBManager.user = nil;
DBManager.level = nil;
DBManager.expand = nil;
DBManager.ad = nil;
--不需要本地存储
DBManager.friend = nil;
DBManager.daily = nil;

function DBManager:ctor()
    local temp_user = require("data.core.DataUser");
    self.user = temp_user.create();
    
    local temp_level = require("data.core.DataLevel");
    self.level = temp_level.create();
    
    local temp_expand = require("data.core.DataExpand");
    self.expand = temp_expand.create();
    
    --friend数据只缓存不存储
    local temp_friend = require("data.core.DataFriend");
    self.friend = temp_friend.create();
    
    --daily数据只缓存不存储
    local temp_daily = require("data.core.DataDaily");
    self.daily = temp_daily.create();
    
    local temp_ad = require("data.core.DataAD");
    self.ad = temp_ad.create();
    
    self.KEY = 
    {
        USER = "userData",
        LEVEL = "levelData",
        EXPAND = "expandData",
        AD = "adData",
        FRIEND = "friendData"
    }
    
    self:init();
    
    self.resetState = false;
    
end

local function debug(...)
    print(string.format(...))
end



function DBManager:init()

    self.uid = getLastId();
    
    debug("dbManager init");
    local data = getData(self.uid);
    
    self:setDataInfo(data);
end

function DBManager:reset()
    self.uid = 1;
    setIntegerForKey("last_id",self.uid);
    flushData();
    self:setDataInfo("");
    self.resetState = true;
end

function DBManager:setDataInfo(info)
    local data = info;
    
    if  data ~= "" 
    --then data = json.decode(data); 
    then data = json.decode(data);
    else data = {};
    end
    
    if(data ==nil or data[self.KEY.USER] == nil)  then 
        data = {}
        data[self.KEY.USER] = {} ;
        data[self.KEY.USER]["uid"] = self.uid;
    end;
    if(data[self.KEY.LEVEL] == nil) then data[self.KEY.LEVEL] = {} end;
    if(data[self.KEY.EXPAND] == nil) then data[self.KEY.EXPAND] = {} end;
    if(data[self.KEY.AD] == nil) then data[self.KEY.AD] = {} end;
    if(data[self.KEY.FRIEND] == nil) then data[self.KEY.FRIEND] = {} end;
    
    self.user:setData(data[self.KEY.USER]);
    self.level:setData(data[self.KEY.LEVEL]);
    self.expand:setData(data[self.KEY.EXPAND]);
    self.ad:setData(data[self.KEY.AD]);
    self.friend:setData(data[self.KEY.FRIEND]);
end

function DBManager:flush()

    local data = {};
    data[self.KEY.USER] = self.user:getData();
    data[self.KEY.LEVEL] = self.level:getData();
    data[self.KEY.EXPAND] = self.expand:getData();
    data[self.KEY.AD] = self.ad:getData();
    data[self.KEY.FRIEND] = self.friend:getData();
    
    setData(self.uid, data);
    flushData();
end

function DBManager:login(uid, pid)
    if uid == nil or tonumber(uid) < 10000000 then return false end
    
    uid = tonumber(uid);
    
    if self.uid ~= uid 
    then 
        if self.uid == 1 then setData("1","") end
        
        self.uid = tonumber(uid);
        
        self:initSwapData(pid);
        
        self.user:setUID(self.uid);
        
        setIntegerForKey("last_id",self.uid);
        self:flush();
        
        MainValue.dataManager.userManager:refresh();
    end
    
    return true;
end

--当两个平台帐号进行切换时需要特殊处理，以免有数据替换的可能。
function DBManager:initSwapData(pid)
    if self.resetState 
    then 
        self.resetState = false;
        local loc_pid = self.user:getPID();
        local loc_name = self.user:getName();
        local loc_picture = self.user:getPicture();
        local loc_username = self.user:getUserName();
        
        local data = getData(self.uid);
        self:setDataInfo(data);
        
        local old_pid = self.user:getPID();

        if old_pid == "" or old_pid == nil
        then
            if loc_pid == pid
            then
                self.user:setPID(loc_pid);
                self.user:setName(loc_name);
                self.user:setPicture(loc_picture);
                self.user:setUserName(loc_username);
            end
        end
    end
end

function DBManager.create()
    local temp = DBManager.new()
    return temp
end

return DBManager;