local DailyManager = class("DailyManager");

DailyManager.data = nil;

function DailyManager:ctor()
    self.data = MainValue.dbManager.daily;
    
    self.gachaFreeCallBack = nil;
    self.gachaBuyCallBack = nil;
    self.gachaOpenAllCallBack = nil;
end

function DailyManager:setdaily(dailyData)
    if dailyData == nil then return ;end
    
    --local daily = dailyData["days"];
    --local signIn = dailyData["signData"];
    --local gacha = dailyData["gachaData"];
    
    self.data:setDaily(dailyData);
    --self.data:setSignIn(signIn);
    --self.data:setGacha(gacha);
    
    MainValue.eventDispatcher:dispatchEvent(MainValue.eventDispatcher.eventType.EVENT_DAILY);
end


-- -------------------------------------------------------------------------SignIn

--[[--
--@return #table
--#table["state"] 今天是否可以领取奖励 (0不可以领取，1可以领)
--#table["totalDays"] 累计签到天数
--#table["list"] 物品配制表
]]
function DailyManager:getSignInList()

    local dailyData = self.data:getDaily();
    
    if dailyData == nil then return nil; end
    
    local configList = dailyData["goods_list"];
    
    if configList == nil then return  end
    
    local loc_list = {};
    local loc_test = {1,2,3,4,5,6,7};
    
    for key, value in ipairs(loc_test) do
        local loc_item = configList[tostring(value)];
        table.insert(loc_list,value,loc_item);
    end
    
    local result = {state = dailyData["state"], totalDays = dailyData["days"], list = loc_list};
    
    return result;
end

function DailyManager:signin()
    local dailyData = self.data:getDaily();
    
    if tonumber(dailyData["state"]) == 1 
    then 
        dailyData["state"] = 0;
        self.data:setDaily(dailyData);
        MainValue.httpManager:dailySignInToServer();
    end
end

--[[--
--@return #table
--#table["date"]  Ymd 年月日
--#table["state"] 今天是否签到 (0未签，1签到过)
--#table["totalDays"] 累计签到天数
--#table["monthDays"] 当月有多少天
--#table["list"] 物品配制表

function DailyManager:getSignInList()
    local signData = self.data:getSignIn();
    
    if signData == nil then return nil; end
    
    local date = signData["date"];
    local state = signData["state"];
    
    local totalDays = signData["d"];
    local configList = signData["goods_list"];
    local monthDays = 30;

    if type(date) == "string" and date~= ""
    then
        local year = string.sub(date,1,4);
        local month = string.sub(date,5,6);
        local day = string.sub(date,7,8);
        monthDays = os.date("%d",os.time({year = string.sub(year,0,4), month = (month) + 1, day = 0}));
    else return nil;
    end
    
    local list = {
                    date = date,
                    state = state, 
                    totalDays = totalDays, 
                    monthDays = monthDays,
                    list = configList;
                 };
                 
    return list;
end
]]

-- -------------------------------------------------------------------------Gacha

--[[--
--获取扭蛋的相关信息
--@return #table
--#table["card"] 每日抽取的8张牌
--#table["num"] 每日抽取的次数(注：此值有时会没有，为nil)
--#table["pos"] 抽取过的礼包ID及位置 = table('礼包ID' = '位置') 当位置标记为1000时，代表这个位置是随机的非固定位置(注：此值有时会没有，为nil)
--#table["list"] 物品配制表
--#table["cash"] 购买扭蛋的cash
--]]
function DailyManager:getGacha()
    local serverData = self.data:getGacha();
    
    if serverData == nil then return nil; end
    
    local dataGacha = {
                    card    = serverData["card"],
                    num     = serverData["num"],
                    pos     = serverData["pos"],
                    cash    = serverData["rand_cash"],
                    list    = serverData["goods"]
    }
    
    return dataGacha;
end

--[[--
--免费抽取
--@pos 位置信息，是1-8的数字
--@callBack 服务器响应后，回调方法
--当位置标记为1000时，代表这个位置是随机的非固定位置
--]]
function DailyManager:freeGacha(pos, callBack)
    if self.gachaFreeCallBack == nil 
    then 
        self.gachaFreeCallBack = callBack;
    else 
        return false;
    end
    
    MainValue.httpManager:gachaFreeToServer(pos);
    
    return true;
end

--[[--
--花钱抽取
--@pos 位置信息，是1-8的数字
--@callBack 服务器响应后，回调方法
--当位置标记为1000时，代表这个位置是随机的非固定位置
--]]
function DailyManager:buyGacha(pos, callBack)
    if self.gachaBuyCallBack == nil 
    then 
        self.gachaBuyCallBack = callBack;
    else 
        return false;
    end

    MainValue.httpManager:gachaBuyToServer(pos);

    return true;
end

--开启所有未抽取的扭蛋
--@callBack 服务器响应后，回调方法
function DailyManager:openAllGacha(callBack)
    if self.gachaOpenAllCallBack == nil 
    then 
        self.gachaOpenAllCallBack = callBack;
    else 
        return false;
    end

    MainValue.httpManager:gachaOpenAllToServer();

    return true;
end

function DailyManager:synchronousGacha(params)

    if params == nil then return nil; end
    
    local serverData = self.data:getGacha();

    if serverData == nil then serverData = {}; end
    
    serverData["card"]= params["card"];
    serverData["num"]= params["num"];
    serverData["pos"]= params["pos"];
    
    self.data:setGacha(serverData);
end

function DailyManager:dailyCallBack(params)
    local _cmd = params["_cmd"];
    
    if _cmd == "free_gacha" then
        if self.gachaFreeCallBack ~= nil then
            self:synchronousGacha(params["gachaData"]);
            self.gachaFreeCallBack(self, params["id"]);
            self.gachaFreeCallBack = nil;
        end
    elseif _cmd == "buy_gacha" then
        if self.gachaBuyCallBack ~= nil then
            self:synchronousGacha(params["gachaData"]);
            self.gachaBuyCallBack(self, params["id"]);
            self.gachaBuyCallBack = nil;
        end
    elseif _cmd == "all_gacha" then
        if self.gachaOpenAllCallBack ~= nil then
            self:synchronousGacha(params["gachaData"]);
            self.gachaOpenAllCallBack(self, params["ids"]);
            self.gachaOpenAllCallBack = nil;
        end
    end
end

function DailyManager.create()
    local temp = DailyManager.new();
    return temp;
end

return DailyManager;