local ExpandManager = class("ExpandManager");

ExpandManager.data = nil;

function ExpandManager:ctor()
    self.data = MainValue.dbManager.expand;
end

--[[--
@param self
@param #boolean isOpen 是否开启背景音乐
]]
function ExpandManager:setBgMusice(isOpen)
    if type(isOpen) ~= "boolean" then return end
    
    self.data:setBgMusice(isOpen);
end

function ExpandManager:getBgMusice()
    return self.data:getBgMusice() ~= false;
end

--[[--
@param self
@param #boolean isOpen 是否开启音效
]]
function ExpandManager:setEfMusice(isOpen)
    if type(isOpen) ~= "boolean" then return end
    
    self.data:setEfMusice(isOpen);
end

function ExpandManager:getEfMusice()
    return self.data:getEfMusice() ~= false;
end

--[[--
--@param id int ftue id
--@return #bool 为true时设置ID成功 ，为false时设置ID不成功
--]]
function ExpandManager:setFtueById(id)
    if id == nil then return end

    local ftue = self.data:getFTUE();
    
    if ftue == nil then ftue = {} end
    
    local exist_ret = self:existFtueById(id);
    
    if exist_ret == true then return false; end
    
    table.insert(ftue,id);
    self.data:setFTUE(ftue);
    
    return true;
end

--[[--
--@return #table 格式[id,id,id]
--]]
function ExpandManager:getFtue()
    local ftue = self.data:getFTUE();
    return ftue;
end

--[[--
--@param id int ftue id
--@return #bool 为true时证明ID已经存在，为false时证明不存在
--]]
function ExpandManager:existFtueById(id)
    local ftue = self.data:getFTUE();

    if ftue == nil then return false; end
    
    for key, i in pairs(ftue) do
    	if i == id then return true; end
    end
    
    return false;
end

--获取能量倒计时时间
function ExpandManager:getEnergyTime()
    return self.data:getEnergyTime();
end

function ExpandManager:setEnergyTime(time)
    self.data:setEnergyTime(time);
end

function ExpandManager:synchronousData(params)
    if params == nil then return ; end
--    if params["firstPlm"] == nil then return ; end
    
    local firstPlm = tonumber(params["firstPlm"]);
    local buyStartPackage = tonumber(params["startpackageid"])
    
    if firstPlm == 1 then
        self.data:setFirstLoginPlatform(1);
    end
    
    if buyStartPackage == 1 then
        self.data:setHasBuyStartPackage(1);
    end
end

--获取是否领取过首次登录信息
--@return true 未领取过，false领取过
function ExpandManager:getFirstLoginPlatform()
    local state = self.data:getFirstLoginPlatform();
    if state == nil then
        return true ;
     else 
        return false; 
     end
end

function ExpandManager:setFirstLoginPlatform()
    local state = self.data:getFirstLoginPlatform();
    if state == nil 
    then 
        self.data:setFirstLoginPlatform(1);
        MainValue.httpManager:firstLoginPlatformToServer();
    end
end

--获取是否买过首充包
--@return true 购买过，false 未购买
function ExpandManager:getHasBuyStartPackage()
    local state = self.data:getHasBuyStartPackage();
    if state == nil then return false ;else return true; end
end

function ExpandManager:setHasBuyStartPackage()
    local state = self.data:getHasBuyStartPackage();
    if state == nil 
    then 
        self.data:setHasBuyStartPackage(1);
        MainValue.httpManager:startPackageHasBuyToServer();
    end
end

--获取是否评论过该游戏
--@return true 评论过，false未评论过
function ExpandManager:getComments()
    local comments = self.data:getComments();
    if comments == nil then return false;
    else return comments; end
end

--设置评论
function ExpandManager:setComments()
    self.data:setComments(true);
end

--设置付费
function ExpandManager:setIsPay()
    self.data:setIsPay(true);
end

function ExpandManager:getIsPay()
    self.data:getIsPay();
end

--是否提示大版本更新
function ExpandManager:setNewVersionStatus()
    self.data:setNewVersionStatus(true);
end

function ExpandManager:getNewVersionStatus()
    return self.data:getNewVersionStatus();
end

function ExpandManager:updateNotifyIOSByLevel(level)
    
end

function ExpandManager:getNotifyIOS()
    --return self.data:getNotifyIOS();
end

function ExpandManager:getResetDay()
    return self.data:getResetDay();
end

function ExpandManager:setResetDay(resetDate)
    self.data:setResetDay(resetDate);
    self.data:setInboxSendHearts(0);
end

function ExpandManager:getStartGameState()
    return self.data:getStartGameState();
end

--设置今日已经在邮箱送过好友心
function ExpandManager:setHasSendInboxHearts()
    self.data:setInboxSendHearts(1);
end

--获取是否在邮箱送过好友心
function ExpandManager:getIsSendInboxHearts()
    if self.data:getInboxSendHearts() == 1 then
        return true;
    end

    return false;
end

--设置游戏开始状态
--@param boolData 布尔值数据，用于表示当前是否已经进入游戏
--(防止打关卡时人为退出不扣心等情况)
function ExpandManager:setStartGameState(boolData)
    self.data:setStartGameState(boolData);
end

function ExpandManager.create()
    local temp = ExpandManager.new();
    return temp;
end

return ExpandManager;