local FriendManager = class("FriendManager");

FriendManager.data = nil;

function FriendManager:ctor()
    self.data = MainValue.dbManager.friend;
    
    self.dailyKey = 
    {
        ENERGY = "energy"
    }
end

--设置已经安装游戏的好友
function FriendManager:setPlatformFriendsInfo(fdData)
    if fdData == nil then return; end
    
    local loc_data = {};
     
    for key, value in pairs(fdData) do
        local loc_id = value["id"];
        loc_data[loc_id] = value;
    end
    
    if next(loc_data) ~= nil then
        self.data:setAllFriend(loc_data); 
    end
end

--设置未安装游戏的好友
function FriendManager:setPlatformInviteFriendsInfo(inviteFriendsData)
    if inviteFriendsData == nil then return; end

    local loc_data = {};

    for key, value in pairs(inviteFriendsData) do
        local loc_id = value["id"];
        loc_data[loc_id] = value;
    end

    if next(loc_data) ~= nil then self.data:setAllInviteFriend(loc_data); end
end

--获取已经安装的好友列表
function FriendManager:getFriends()
    return self.data:getAllFriend();
end

--获取未安装的好友列表
function FriendManager:getInviteFriends()
    return self.data:getAllInviteFriend();
end

--根据好友id获取头像地址
function FriendManager:getPictureById(id)
    if self.data:getAllFriend()[id] ~= nil then
        return self.data:getAllFriend()[id].picture;
    else
        return nil;
    end
end

function FriendManager:getCurrentLevelFriends(allFriend, lv)
    if allFriend == nil then return nil; end
    lv = tostring(lv);
    local loc_data = allFriend;
    
    if loc_data == nil then return nil end;
    
    local loc_friends = {};
    
    for key, value in pairs(loc_data) do
        local loc_lvData = value['lvData'];
        
        if loc_lvData ~= nil and loc_lvData[lv] ~= nil
        then
            local loc_id = value["id"];
            local loc_energy = 0;
            if value[self.dailyKey.ENERGY] ~= nil
            then
                loc_energy = value[self.dailyKey.ENERGY];
            end
            
            local temp_friends = {
                id = value["id"],
                name = value["name"],
                username = value["username"],
                picture = value["picture"],
                score = loc_lvData[lv]["score"],
                stars = loc_lvData[lv]["stars"],
                energy = loc_energy,
            }
            
            table.insert(loc_friends,temp_friends);
        end
    end
    
    if next(loc_friends) ~= nil then return loc_friends; end
    
    return nil;
end


--[[--
获取排序
--@param lv
--@param nsf 是否把用户自己去掉，true去掉， 不传参数或者为nil则不去掉
]]--
function FriendManager:getRankFriends(lv, nsf)
    local loc_data = self.data:getAllFriend();
    local loc_currentData = self:getCurrentLevelFriends(loc_data,lv) or {};
    if nsf==nil then
        local user_score, user_stars = MainValue.dataManager.levelManager:getLevel(lv)
        local loc_user = {
            ["id"]       = MainValue.dataManager.userManager:getPID(),
            ["name"]     = MainValue.dataManager.userManager:getName(),
            ["username"] = MainValue.dataManager.userManager:getUserName(),
            ["picture"]  = MainValue.dataManager.userManager:getPicture(),
            
            ["score"]    = user_score,
            ["stars"]    = user_stars
        }
        print("loc_user.picture110")
        print(loc_user.picture)
        print("loc_user.picture120")
        table.insert(loc_currentData, loc_user)
    end
    
    if loc_currentData~=nil then
        table.sort(loc_currentData, function (u1, u2)
            return tonumber(u1["score"]) > tonumber(u2["score"])
        end)
    end
    return loc_currentData;
end

--超越最大分数
function FriendManager:getBeyondScoreFriData(lv, old_score, new_score)

    if old_score>=new_score then return nil end
    local loc_currentData = self:getRankFriends(lv, true)
    if loc_currentData == nil then return nil; end
    local u_newSeq, u_oldSeq
    for i, val in ipairs(loc_currentData) do
        if new_score> tonumber(val["score"]) then
            u_newSeq = u_newSeq or i
            if old_score> tonumber(val["score"]) then
    			u_oldSeq = i
                break
    		end
    	end
    end
    --if u_newSeq~=nil and u_oldSeq~=nil and u_oldSeq>u_newSeq then
    if u_newSeq~=nil then
        return loc_currentData[u_newSeq] or nil
    else
        return nil
    end
end


--超越最大关卡
function FriendManager:getBeyondMaxlvFriData(lv)
    local loc_data = self.data:getAllFriend();
    if loc_data == nil then return nil; end
    local mlv = MainValue.mxUIManager.mapManager:getMaxLevel();
    
    if lv == mlv then 
        return nil;
    end
    
    for i, val in pairs(loc_data) do
        if val["maxLevel"] ~= nil and tonumber(val["maxLevel"]) == lv then
            return val
        end
    end
    return nil
end


--获取好友id列表
function FriendManager:getFriendsIds()
    local loc_data = self.data:getAllFriend();
    if loc_data == nil then return nil; end
    
    local loc_ids = {};
    
    for key, value in pairs(loc_data) do
        table.insert(loc_ids,key);
    end
    
    if next(loc_ids) == nil
    then return nil;
    else return loc_ids;
    end
end

--获取随机五名好友，用于邮箱送心
function FriendManager:getRandomFiveFriendsIds()
    local loc_data = self:getFriendsIds();
    if loc_data == nil then return nil; end
    
    local idsList = {};
    
    if #loc_data <= 5 then
        idsList = loc_data;
    else
        for i= 1,5 do
            local len = #loc_data;
            local randomIndex = math.random(1, len);
            table.insert(idsList, loc_data[randomIndex]);
            table.remove(loc_data, randomIndex)
        end
    end
    return idsList;
end

-- 融合服务器的数据
function FriendManager:synchronousFriends(fdData)

    if fdData == nil then return false; end
    
    local loc_data = self.data:getAllFriend();
    if loc_data == nil then return false; end
    
    for key, value in pairs(loc_data) do
        local loc_id = tostring(key);
    	
    	if loc_data[loc_id] ~= nil then
            loc_data[loc_id][self.dailyKey.ENERGY] = self:getDaily(loc_id, self.dailyKey.ENERGY);
        end
        
    	if fdData[loc_id] ~= nil
    	then
            loc_data[loc_id]['maxLevel'] = fdData[loc_id]["max_lv"];
            loc_data[loc_id]['lvData'] = fdData[loc_id]["lvData"];
            
            local maxStar = 0;
            if fdData[loc_id]["lvData"]~=nil then
                for fkey, fvar in pairs(fdData[loc_id]["lvData"]) do
                    if fvar["stars"] ~= nil then
                        maxStar = maxStar + tonumber(fvar["stars"]);
                    end
                end
            end
            loc_data[loc_id]['maxStar'] = maxStar;
    	end
    end
    
    self.data:setAllFriend(loc_data);
    
    MainValue.eventDispatcher:dispatchEvent(MainValue.eventDispatcher.eventType.EVENT_FRIENDS, loc_data);

    
    return true;
end

--暂时废弃
--[[
function FriendManager:setFriendSendGift(pid)
    if pid == nil then return false; end
    
    pid = tostring(pid);
    if pid == "" then return false; end
    
    local pfdata = self.data:getAllPlatform();
    local loc_data = self.data:getAllFriend();
    
    if loc_data[pid] == nil then return false; end
    
    local is_set = false;
    
    
    if pfdata == nil
    then
        is_set = true;
        pfdata = {};
        pfdata[pid] = {send=1};
    else
    
        if pfdata[pid] == nil
        then
            is_set = true;
            pfdata[pid] = {send=1};
        else
            if pfdata[pid]["send"] == nil
            then
                is_set = true;
                pfdata[pid]["send"] = 1;
            end--endif
        end--endif
    --end--endif
    
    if is_set
    then
        loc_data[pid]["send"] = 1;
        self.data:setAllPlatform(pfdata);
        self.data:setAllFriend(loc_data);
        return true;
    end
    
    return false;
end

--暂时废弃
function FriendManager:setFriendAskGift(pids)
    if pids == nil or type(pids) ~= "table" or next(pids) == nil then return false; end

    pid = tostring(pid);
    if pid == "" then return false; end

    local pfdata = self.data:getAllPlatform();
    local loc_data = self.data:getAllFriend();

    if loc_data[pid] == nil then return false; end

    local is_set = false;

    
    if pfdata == nil
    then
    is_set = true;
    pfdata = {};
    pfdata[pid] = {send=1};
    else
    
    if pfdata[pid] == nil
    then
        is_set = true;
        pfdata[pid] = {send=1};
    else
        if pfdata[pid]["send"] == nil
        then
            is_set = true;
            pfdata[pid]["send"] = 1;
        end--endif
    end--endif
    end--endif

    if is_set
    then
        loc_data[pid]["send"] = 1;
        self.data:setAllPlatform(pfdata);
        self.data:setAllFriend(loc_data);
        return true;
    end

    return false;
    
end
]]

--排序好友
function FriendManager:getOrderlyFriends()
    local loc_data = self.data:getAllFriend();
    if loc_data == nil then return nil; end
    local cur_uid = MainValue.dataManager.userManager:getPID();
    if cur_uid == nil or cur_uid == "" then return nil; end
    
    local allOrderFriends = {};
    -- 格式化数据
    for akey, avar in pairs(loc_data) do
        if avar["id"]~=nil and avar["id"]~="" then
            avar["maxLevel"] = avar["maxLevel"] or 0;
            avar["maxStar"] = avar["maxStar"] or 0;
            table.insert(allOrderFriends,avar);
        end
    end
    local loc_user = {
        ["id"]       = MainValue.dataManager.userManager:getPID(),
        ["name"]     = MainValue.dataManager.userManager:getName(),
        ["username"] = MainValue.dataManager.userManager:getUserName(),
        ["picture"]  = MainValue.dataManager.userManager:getPicture(),
        ["maxLevel"] = MainValue.dataManager.levelManager.getMaxLevel(),
        ["maxStar"]  = MainValue.dataManager.levelManager.getMaxStar()
    }
    -- 将当前用户插入
    table.insert(allOrderFriends,loc_user);
    
    -- 按level和star排序
    table.sort(allOrderFriends, function(a, b)
        if tonumber(a["maxLevel"]) > tonumber(b["maxLevel"]) then
            return true
        elseif tonumber(a["maxLevel"]) == tonumber(b["maxLevel"]) then
            return tonumber(a["maxStar"]) > tonumber(b["maxStar"])
        else
            return false
        end
    end)
    
    if next(allOrderFriends) == nil then 
        return nil;
    else
        local rs_data = {} -- 返回前十名
        
        local i = 1
        local cur_user_seq = nil; -- 当前用户排名
        
        --取得用户排名
        while allOrderFriends[i] do
            if cur_uid==allOrderFriends[i]["id"] then
                cur_user_seq = i
                break 
            else
                i = i+1
            end
        end
        
        -- 取前十
        for i=1, 10 do  
            table.insert(rs_data, allOrderFriends[i]);
        end 
        
        return rs_data , cur_user_seq;
    end

end

-- -----------------------------------------------------------------------------inbox

function FriendManager:synchronousInbox(inboxData)
    if inboxData == nil then return false; end
    self.data:setInbox(inboxData);
end

--获取信箱数据
function FriendManager:getInbox()
    local inboxData = self.data:getInbox();
    if inboxData == nil then return {}; 
    else return inboxData; end
end

--删除信箱邮件
function FriendManager:delInbox(request_id)
    if request_id == nil or request_id == "" then return false; end
    
    request_id = tostring(request_id);
    local inboxData = self.data:getInbox();
    
    if inboxData[request_id] ~= nil 
    then
        inboxData[request_id] = nil;
        
        MainValue.httpManager:deleteInboxToServer(request_id);
        
        return true;
    end
    
    return false;
end

-- -----------------------------------------------------------------------------daily

--获取发送好友送心等限制
function FriendManager:getDaily(id, dailyKey)
    if id == nil or dailyKey == nil then return 0; end

    id = tostring(id);
    local loc_data = self.data:getDaily();
    local loc_key = tostring(dailyKey);
    
    if loc_data == nil then return 0; end
    
    for key, var in pairs(loc_data) do
        local fdid = tostring(key);
        local loc_daily = 0;
        
        if var == nil 
        then 
            loc_daily = 0;
        elseif var[loc_key] ~= nil
        then
            loc_daily = tonumber(var[loc_key]);
        end
        
        if fdid == id
    	then
            return loc_daily;
    	end
    end
    
    return 0;
end

--设置发送好友送心限制(第天一次)
function FriendManager:setDailyEnergyById(id)
    if id == nil then return false; end
    
    id = tostring(id);
    
    local loc_fdData = self.data:getAllFriend();
    if loc_fdData == nil then return false; end
    if loc_fdData[id] == nil then return false; end
    
    local loc_data = self.data:getDaily();
    local loc_key = self.dailyKey.ENERGY;

    if loc_data == nil then loc_data = {}; end
    
    if loc_data[id] == nil
    then
        loc_data[id] = {};
        loc_data[id][loc_key] = 1;
    else 
        loc_data[id][loc_key] = 1;
    end
    
    self.data:setDaily(loc_data);
    
    --好友列表同步更新
    loc_fdData[id][loc_key] = 1;
    self.data:setAllFriend(loc_fdData);
    
    return true;
end

function FriendManager:reset()
    self.data:setDaily(nil);
end

function FriendManager.create()
    local temp = FriendManager.new();
    return temp;
end

return FriendManager;