local GoodsManager = class("GoodsManager");

GoodsManager.data = nil;

function GoodsManager:ctor()
    self.data = MainValue.dbManager.expand
end

function GoodsManager:getAllGoods()
	return self.data:getAllGoods(); 
end

---------------------------
--@param self
--@param #int id 道具编号
--@param #int num 增加的数量
--@return #int 道具数量
function GoodsManager:addGoods(id,num)
    if id == nil or num == nil then return 0; end

    local goods = self.data:getGoods(id);
    if goods == nil then
        if num > 0 then
            self.data:setGoods(id, num);
            MainValue.eventDispatcher:dispatchEvent(MainValue.eventDispatcher.eventType.EVENT_GOODS, id, num);
            return num;
        else
            return 0;
        end
    else 
        local newNumber = goods + num
        
        if newNumber < 0 then
            newNumber = 0;
        end
        
        self.data:setGoods(id, newNumber);
        MainValue.eventDispatcher:dispatchEvent(MainValue.eventDispatcher.eventType.EVENT_GOODS, id, newNumber);
        return newNumber;
    end
end

---------------------------
--@param self
--@param #int id 道具编号
--@param #int num 减少的数量
--@return #int 道具数量
function GoodsManager:subGoods(id,num)
    if id == nil or num == nil or tonumber(num) <= 0 then return 0; end
    
    local goods = self.data:getGoods(id);
    if goods == nil then
        if num > 0 then
            self.data:setGoods(id, num);
            MainValue.eventDispatcher:dispatchEvent(MainValue.eventDispatcher.eventType.EVENT_GOODS, id, num);
            return num
        else
            return 0
        end
    else 
        local newNumber = goods - num
        
        if newNumber < 0 then
            newNumber = 0
        end
        
        self.data:setGoods(id, newNumber);
        MainValue.eventDispatcher:dispatchEvent(MainValue.eventDispatcher.eventType.EVENT_GOODS, id, newNumber);
        return newNumber;
    end
end

---------------------------
--@param #int id 道具编号
--@return #int 道具数量
function GoodsManager:getGoodsNumber(id)
    local goodsNumber = self.data:getGoods(id);
    if goodsNumber == nil then
        return 0
    else 
        return goodsNumber > 0 and goodsNumber or 0
    end
end

--@param lvPar   判断是否有道具要解锁 根据解锁级别 返回要解锁的道具数据属性
function GoodsManager:havePropIsUnlockingByLv(lvPar)
    if lvPar == nil or lvPar <= 0 then return nil; end

    local tempropsDataObj = nil
    
    local goodsConfig = MainValue.infosManager:getObjGoods();
    
    if goodsConfig == nil then return nil; end
    
    for key, var in pairs(goodsConfig) do
        if tonumber(var['lv']) == lvPar and tonumber(var['type']) == 1 then
            local isorNotUnlock = GoodsManager:isOrNotAlreadyUnlock(var['id'])
            if not isorNotUnlock then  
                tempropsDataObj = var
                self:setUnLockList(var)
                break
            elseif isorNotUnlock then
                break
            end           
        end
    end
    
    return tempropsDataObj
end

--@param propIdPar 根据游戏内部的道具id 判断是否已经被解锁了
function GoodsManager:isOrNotAlreadyUnlock(propIdPar)
    local isUnlock = false
    local unlockList = self:getUnLockList()
        
    for propI = 1,#(unlockList) do
        if unlockList[propI] == tonumber(propIdPar) then
            isUnlock = true
            break
        end
    end    
	return isUnlock
end


function GoodsManager:getUnLockList()
    self.data = MainValue.dbManager.expand   --TODO::: 应该有其他地方又 实例化goodsManager造成的  TODO:::
    local unLockData = self.data:getUnLockGoods()
    if unLockData == nil then unLockData = {}; end
    return unLockData;
end

--@private
function GoodsManager:setUnLockList(itemData)
    if type(itemData) ~= "table" then return false; end
    
    if itemData["id"] == nil then return false; end
    
    local id = tonumber(itemData["id"]);
    
    if self:existUnLockList(id) == false
    then
        local unLockData = self.data:getUnLockGoods();
        
        if unLockData == nil then unLockData = {id};
        else table.insert(unLockData,id); end
        
        ----------2015.2.9 lg
--        self:addGoods(id, 2);
        self.data:setUnLockGoods(unLockData);
        
        return true;
    end
    
    return false;
end

--[[
--判断是否重复
--@public
--@params id number 需要解锁的ID
]]
function GoodsManager:existUnLockList(id)
    if type(id) ~= "number" or id <= 0 then return true; end
    
    local unLockData = self.data:getUnLockGoods();

    if unLockData == nil then return false; end

    for key, i in pairs(unLockData) do
        if i == id then return true; end
    end

    return false;
end


function GoodsManager.create()
    local temp = GoodsManager.new();
    return temp;
end

return GoodsManager;