local LevelManager = class("LevelManager");

local maxLevel = nil;
local maxStar = nil;

--@return #int levelData 的长度, #star 所有关卡的星星数
local function getAllLevelLength(self)
    local count = 0;
    local star = 0;
    local allLevel = self.data:getAllLevel();
    
    for k, v in pairs(allLevel) do
        count = count + 1
        
        if v ~= nil and v["stars"] ~= nil 
        then
            star = star + tonumber(v["stars"]);
        end
    end

    return count, star;
end

LevelManager.data = nil;

function LevelManager:ctor()
    self.data = MainValue.dbManager.level;
    maxLevel, maxStar = getAllLevelLength(self);
end

--同步关卡数据
--@param self type description

function LevelManager:synchronousLevel(lvData)
    
    if lvData == nil then return false end
    
    local loc_data = {};
    
    for key, value in pairs(lvData) do
        local lv = tostring(key);
        local star = tonumber(value['stars']);
        local score = tonumber(value['score']);
        
        local loc_comret = self:levelComparisons(lv,star,score,1);
        
        loc_data[lv] = loc_comret.lvData;
        
    end
    
    if next(loc_data) ~= nil 
    then 
        self.data:setAllLevel(loc_data); 
        maxLevel, maxStar = getAllLevelLength(self);
        MainValue.eventDispatcher:dispatchEvent(MainValue.eventDispatcher.eventType.EVENT_LEVEL, loc_data);
    end
    
end


--[[--
保存关卡数据
--@param self 
--@param lv 关卡编号
--@param stars 评级 {{start:1,score:100},{start:2,score:200},{start:3,score:300}}
--@param score 得分
--@param rt 是否成功 true:false
]]
function LevelManager:saveLevel(lv,stars,score,rt)
    lv = tostring(lv);
    local star = 0
   
    
    rt = tonumber(rt);
    star = tonumber(stars);
    score = tonumber(score);
    
    local loc_comret = self:levelComparisons(lv,star,score,rt);
    
    if loc_comret.isUpData then
        self.data:setLevel(lv, loc_comret.lvData);
        maxLevel, maxStar = getAllLevelLength(self);
        --向服务器保存关卡数据
        MainValue.httpManager:saveLevelToServer(lv, rt, score, star);
    end
end

function LevelManager:levelComparisons(lv,stars,score,rt)
    --更新内存数据
    --数据是否发生变化
    local isUpData = false;
    local lvData = self.data:getLevel(lv);
    local star = stars;
    if score == nil then score = 0 end;
    if star == nil then star = 0 end;
    
    --新记录
    local newLv = {score = score, stars = star};
    local oldLv = lvData;
    
    
    --记录不存在
    if oldLv == nil then 
        isUpData = true;
        lvData = newLv;
    else
        if oldLv.stars == nil then lvData.stars = 0 end;
        if oldLv.score == nil then lvData.score = 0 end;
        if newLv.stars ~= oldLv.stars or newLv.score ~= oldLv.score then
            isUpData = true
            if tonumber(newLv.stars) > tonumber(oldLv.stars) then
                lvData["stars"] = tonumber(newLv.stars);
            end

            if tonumber(newLv.score) > tonumber(oldLv.score) then
                lvData["score"] = tonumber(newLv.score);
            end
        end
    end
    
    return {isUpData = isUpData, lvData = lvData};
end

--[[--
获取关卡数据表
--@param self
--@return #table 例如 "1":{"score":1000,"stars":1},"2":{"score":2000,"stars":2}
]]
function LevelManager:getAllLevel()
    return self.data:getAllLevel();
end

--[[--
获取指定关卡得分数据
--@param lv 关卡编号
--@return 参数1：score,参数2：stars,如果关卡不存在，则返回两个值都为0
--使用：local a,b = self:getLevel(1)
]]
function LevelManager:getLevel(lv)
    local lv = self.data:getLevel(lv);
    if lv == nil then
        return 0,0
    end
    return lv.score,lv.stars
end

function LevelManager:getMaxLevel()
    return maxLevel+1;
end

function LevelManager:getMaxStar()
    return maxStar;
end

-- ---------------------------------------------------------------------------------BOX

function LevelManager:getAllBox()
    return self.data:getAllBox();
end

--[[--
--@param bid int box id
--@return #bool 返回数据存储是否正常
]]
function LevelManager:openBoxById(bid)
    local data = self.data:getBox(bid);

    if data == nil
    then 
        self.data:setBox(bid, 1);
        MainValue.httpManager:openBoxGiftToServer(bid);
        return true;
    end

    return false;
end

function LevelManager:synchronousBox(boxData)
    if boxData == nil then return false end
    self.data:setAllBox(boxData);
end

function LevelManager:existBoxId(bid)
    local data = self.data:getBox(bid);
    if data == nil then return true; end
    return false;
end

-- ---------------------------------------------------------------------------------BLOCK
--[[
function LevelManager:getAllBlock()
    return self.data:getAllBlock();
end


--@param bid int blockID，就是当前BLOCK前面的关卡数
--@return #bool 返回数据存储是否正常

function LevelManager:openBlockById(bid)
    local data = self.data:getBlock(bid);
    
    if data == nil
    then 
        self.data:setBlock(bid, {bk=0});
        return true;
    end
    
    return false;
end

local function existBlockFids(fids)
    local ids = {};
    
    for key, var in pairs(fids) do
        var = tostring(var);
        
    	if tostring(var) == "1000" 
    	then
    	   table.insert(ids,var);
    	else
    	   local rep = true;
    	   for new_key, new_var in pairs(ids) do
                new_var = tostring(new_var);
    	       if new_var == var 
    	       then 
    	           rep = false;
    	           break;
    	       end
    	   end
    	   
    	   if rep then table.insert(ids,var); end
    	end
    end
    
    return ids;
end


--@param bid int blockID，就是当前BLOCK前面的关卡数
--@param fids array 帮助BLOCK的朋友ID，花钱买时ID为1000
--@return #bool 返回数据存储是否正常

function LevelManager:setBlockFID(bid, fids)

    if(fids == nil) then return false end
    
    fids = existBlockFids(fids);
    
    local loc_data = self.data:getBlock(bid);

    if loc_data ~= nil
    then 
        if loc_data.bk == 1 then return true end
        if loc_data.ids == nil then loc_data.ids = {} end
        
        for key,value in ipairs(fids) do
            value = tostring(value);
            if value ~= "1000"
            then
                local loc_repeat = true;
                for old_key, old_value in ipairs(loc_data.ids) do
                    if value == old_value then loc_repeat = false; end
                end
                
                if loc_repeat == true  then table.insert(loc_data.ids,value); 
                else return false; end
            else
                table.insert(loc_data.ids,value);
            end
        end
        
        local loc_isBlock = self:isBlock(bid, loc_data);
        
        if loc_isBlock then loc_data.bk = 1 end
        
        self.data:setBlock(bid, loc_data);
        
        return true;
    end

    return false;
end

function LevelManager:isBlock(bid, bkData)
    -- 判断是否全部解锁
    local loc_data = bkData;
    local loc_bkData = mapConfig.blocker[MainValue.mxUIManager.mapManager:getBlockerIndexByMissionID(bid)];
    
    if loc_bkData ~= nil 
    then
        local loc_bid = loc_bkData["act"];
        if loc_bid ~= bid then return false end
        
        local loc_count = 0;
        local loc_maxCount = loc_bkData["ct"];
        
        for oldid in ipairs(loc_data.ids) do
            loc_count = loc_count + 1;
        end
        
        if loc_count >= loc_maxCount then return true end
    end
    
    return false;
end

function LevelManager:existBockFids()

end

function LevelManager:synchronousBlock(bkData)

    if bkData == nil then return false end
    
    local loc_serverData = bkData['data'];
    if loc_serverData == nil then return false; end
    
    self.data:setAllBlock(loc_serverData);
end
]]

function LevelManager.create()
    local temp = LevelManager.new();
    return temp;
end

return LevelManager;