local UserManager = class("UserManager");

UserManager.data = nil;

function UserManager:ctor()
    self.data = MainValue.dbManager.user;
end

-- -----------------------------------------------------------------------------me

-- 获取服务器生成的UID
function UserManager:getUID()
    return self.data:getUID();
end

-- 获取平台ID
function UserManager:getPID()
    return self.data:getPID();
end

-- 例子：Zhongbao  Gan
function UserManager:getName()
    return self.data:getName();
end

-- 例子：Zhongbao  (后面不带姓氏，只有名字)
function UserManager:getUserName()
    return self.data:getUserName();
end

-- 获取自己在平台的头像
function UserManager:getPicture()
    return self.data:getPicture();
end

function UserManager:setPlatformInfo(pinfo)
    if pinfo == nil then return false; end
    
    self.data:setName(pinfo["name"]);
    self.data:setUserName(pinfo["username"]);
    self.data:setPicture(pinfo["picture"]);
    self.data:setPID(pinfo["id"]);
    
    MainValue.dbManager:flush();
    MainValue.eventDispatcher:dispatchEvent(MainValue.eventDispatcher.eventType.EVENT_PLATFORM);
end

-- -----------------------------------------------------------------------------coin

--[[--
增加金币
--@param self
--@param num 增加的金币数量

function UserManager:addCoin(num)
    if num <= 0 then return false end
    
    local old_coin = self.data:getCoin();
    
    local _coin = old_coin + num;
    
    if _coin < 0 then
        _coin = 0;
    end
    
    self.data:setCoin(_coin);
    MainValue.eventDispatcher:dispatchEvent(MainValue.eventDispatcher.eventType.EVENT_COIN);
    
    return true;
end

function UserManager:subCoin(num)
    local old_coin = self.data:getCoin();
    
    if old_coin < num or old_coin <= 0
    then
    return false;
    end
    
    local _coin = old_coin - num;
    
    self.data:setCoin(_coin);
    MainValue.eventDispatcher:dispatchEvent(MainValue.eventDispatcher.eventType.EVENT_COIN);

    return true;
end

function UserManager:getCoin()
    return self.data:getCoin();
end
]]
-- -----------------------------------------------------------------------------cash

--[[--
增加现金
--@param num number 增加的现金数量
]]--
function UserManager:addCash(num)
    if num <= 0 then return false end
    
    local old_cash = self.data:getCash();
    
    local _cash = old_cash + num;
    
    if _cash < 0 then
        _cash = 0;
    end

    self.data:setCash(_cash);
    MainValue.eventDispatcher:dispatchEvent(MainValue.eventDispatcher.eventType.EVENT_CASH);
    
    return true;
end

--[[--
扣除现金
--@param self
--@param num  扣除的现金数量
]]--
function UserManager:subCash(num)

    local old_cash = self.data:getCash();
    
    if old_cash < num or old_cash <= 0
    then
        return false;
    end
    
    local _cash = old_cash - num;

    self.data:setCash(_cash);
    MainValue.eventDispatcher:dispatchEvent(MainValue.eventDispatcher.eventType.EVENT_CASH);
    
    return true;
end

function UserManager:getCash()
    return self.data:getCash();
end

-- -----------------------------------------------------------------------------heart

--[[--
增加心
--@param num number 增加的心数量
]]--
function UserManager:addHeart(num)

    local old_heart = self.data:getHeart();
    local _heart = old_heart + num;
    if _heart < 0 then
        _heart = 0;
    end
    
    self.data:setHeart(_heart);
    MainValue.eventDispatcher:dispatchEvent(MainValue.eventDispatcher.eventType.EVENT_HEART);
    
    
    self:setEnergyTime();
    
    return _heart;
end

--[[--
扣除心
--@param self
--@param num  扣除的心数量
]]--
function UserManager:subHeart(num)

    local old_heart = self.data:getHeart();
    local _heart = old_heart - num;
    if _heart < 0 then
        _heart = 0;
    end

    self.data:setHeart(_heart);
    MainValue.eventDispatcher:dispatchEvent(MainValue.eventDispatcher.eventType.EVENT_HEART);
    
    if _heart == 4 then --从满心状态扣一颗心，记录时间戳
        self:setEnergyTime();
    end
    
    return _heart;
end

-- 保存体力变化时间戳
function UserManager:setEnergyTime()
    local stamp = os.time();
    MainValue.dataManager.expandManager:setEnergyTime(stamp);
end

function UserManager:getHeart()
    return self.data:getHeart();
end

function UserManager:refresh()
    MainValue.eventDispatcher:dispatchEvent(MainValue.eventDispatcher.eventType.EVENT_COIN);
    MainValue.eventDispatcher:dispatchEvent(MainValue.eventDispatcher.eventType.EVENT_CASH);
    MainValue.eventDispatcher:dispatchEvent(MainValue.eventDispatcher.eventType.EVENT_HEART);
end

function UserManager.create()
    local temp = UserManager.new();
    return temp;
end

return UserManager;