local EventDispatcher = class("EventDispatcher");

--可以注册的回调函数类型
EventDispatcher.eventType = { 
    EVENT_COIN = 1,
    EVENT_CASH = 2,
    EVENT_HEART = 3,
    EVENT_GOODS = 4,
    EVENT_FRIENDS = 5, --platform好友数据与服务器的数据融合后
    EVENT_LEVEL = 6, --融合完服务器的关卡数据后
    EVENT_PLATFORM = 7, --获取到自己的平台信息
    EVENT_DAILY = 8 --获取到daily信息
}

function EventDispatcher:ctor()

    --各模块的回调函数表
    self.callBackFuncData = {}

end

---
--@param self
--@param #function 函数
--@param #int 回调类型：
function EventDispatcher:addEventListener(_type, callBack)
    if type(_type) == "number" and _type < self.eventType.EVENT_COIN then return end
    if self.callBackFuncData[_type] == nil then
        self.callBackFuncData[_type] = {}
	end
    table.insert(self.callBackFuncData[_type],callBack)
end

---
--@param self
--@param #int 回调类型：
--@param #function 函数
function EventDispatcher:removeEventListener(_type, callBack)
    if type(_type) == "number" and _type < self.eventType.EVENT_COIN then return end
    
    local funcTable = self.callBackFuncData[_type];
    if funcTable ~= nil then
        for i, func in pairs(funcTable) do
            if func == callBack then
				table.remove(funcTable, i)
            end
        end
        
        if next(funcTable) == nil then
            self.callBackFuncData[_type] = nil;
        end
    end
end

---
--执行回调函数
--@param self
--@param DataManager.eventType 事件类型
--@param 自定义参数
--@param 自定义参数
--@param 同上...
function EventDispatcher:dispatchEvent(...)
    local arg = {...}
    if arg == nil or #arg == 0 then
		return
	end
    local funcTable = self.callBackFuncData[arg[1]]
    if funcTable ~= nil then
        table.foreach(funcTable, function(i, func)
            func(arg[2],arg[3])
        end)
    end
end

function EventDispatcher.create()
    local temp = EventDispatcher.new();
    return temp;
end

return EventDispatcher;