require "game.GameValue";
require "ui.playGamePanel.PlayGamePanel"

local GameStatusManager = class("GameStatusManager");

function GameStatusManager:ctor()
    self.winScore = nil; --胜利分数
    self.bestScore = nil; --历史最好得分
    self.starData = nil; --当前关卡星星配置数据
    self.loseData = nil; --失败原因数据
    
    self.exitType = nil; --退出类型（win or lose）
    
    self.targetData = nil; --收集目标数据
    
    self.panelCount = 0; --胜利面板后需要显示面板数量
    
    self.loadingPanel = nil; --连接FB的loading面板
    
    self.needCloseFBConnect = true; --是否需要关闭FBConnectPanel
end

function GameStatusManager.create()
    local temp = GameStatusManager.new();
    
    return temp;
end

--[[保存胜利数据]]
function GameStatusManager:setWin(scores, starData)
    self.winScore = scores;
    self.starData = starData;
    self.oldScore = MainValue.dataManager.levelManager:getLevel(MainValue.SELECTED_LEVEL);

    self.exitType = GameValue.GAME_RESULT_TYPE_ONE;
end

--[[保存失败数据]]
function GameStatusManager:setLose(loseData)
    MainValue.dataManager.userManager:subHeart(1);--失败扣心
    
    self.loseData = loseData;
    
    self.exitType = GameValue.GAME_RESULT_TYPE_TWO;
end

--[[退出游戏，进入地图]]
function GameStatusManager:onQuitMatchGame()
    if self.exitType == nil then
        return ;
    end
    
    if self.exitType == GameValue.GAME_RESULT_TYPE_ONE then
    
        if MainValue.mxUIManager.mapManager:getMaxLevel() == MainValue.SELECTED_LEVEL and MainValue.dataManager.levelManager:getMaxLevel() == MainValue.SELECTED_LEVEL then 
            MainValue.adsManager:showVideoAdsByCompany(2);
        else
            MainValue.adsManager:interstitialAd() 
        end
    
        MainValue.mxUIManager:showWinPanel(self.winScore, self.oldScore, self.starData);
        
    elseif self.exitType == GameValue.GAME_RESULT_TYPE_TWO then
        MainValue.mxUIManager:showOverPanel(self.loseData);
        
    end
    
    self.exitType = nil;
end

--[[判断胜利面板后续面板是否显示完毕]]--
function GameStatusManager:setShowPanelCount(count, isPlus)
    isPlus = isPlus or false;
    
    if isPlus then
        self.panelCount = self.panelCount + count;
    else
        self.panelCount = self.panelCount - count;
        
        if self.panelCount <= 0 then --后续面板显示完毕，头像开始走蚂蚁线
            self:hasNewLevel();
            self.panelCount = 0;
        end
    end
end

--显示连接FB的loading
function GameStatusManager:showConnectFBLoading()
--    if MainValue.mxUIManager.state == 1 then
    local FBConnectCls = require("ui/FBConnect/FBConnectPanel")
    self.loadingPanel = FBConnectCls.create();
    self.loadingPanel:appearAlone();

    self.needCloseFBConnect = true;
--    end
end

--关闭loading显示成功
function GameStatusManager:showConnectFBSuccess(isSuccesss)
    if self.loadingPanel == nil then
        return;
    end

    if self.needCloseFBConnect or not isSuccesss then
        self.loadingPanel:disapperAlone(isSuccesss);
    else
        local FBConnectSuccessClz = require("ui.FBConnect.FBConnectSuccessPanel")
        local FBConnectSuccess = FBConnectSuccessClz.create();
        FBConnectSuccess:appearAlone();

        self.needCloseFBConnect = true;
    end
end

function GameStatusManager:setNeedCloseFBConnect(boo)
    self.needCloseFBConnect = boo;
end

--打开宝箱面板
function GameStatusManager:openChests()
    if MainValue.SELECTED_LEVEL % 10 == 0 then
        local pbcInfo = mapConfig.box[MainValue.SELECTED_LEVEL/10];
        local blockerLockcls = require("ui/boxLock/BoxLockPanel");
        local blockerLock = blockerLockcls.create(pbcInfo,1);
        blockerLock:appear();
    end
end

--[[检验下一关]]
function GameStatusManager:hasNewLevel()
--    if MainValue.SELECTED_LEVEL % 10 == 0 and not MainValue.mxUIManager.mapManager:isSuccesBlocker(MainValue.SELECTED_LEVEL) then
--        return;
--    end
    
--    if MainValue.SELECTED_LEVEL ~= MainValue.dataManager.levelManager:getMaxLevel() and  MainValue.SELECTED_LEVEL < MainValue.mxUIManager.mapManager:getMaxLevel() then
--        MainValue.mxUIManager.mapManager:upDateMission(MainValue.SELECTED_LEVEL);
--    end
    MainValue.mxUIManager.mapManager:upDateMission(MainValue.SELECTED_LEVEL);
end

--[[开启下一关面板]]
function GameStatusManager:openNewLevel()
    
    --弹出5星评论
    local newLevel = MainValue.dataManager.levelManager:getMaxLevel();
    
    if newLevel > MainValue.mxUIManager.mapManager:getMaxLevel() then
        return;
    end
    
    
--    if newLevel == ad.maxLevel + 1 then
--        MainValue.mxUIManager:addVideoEgg();
--    end;


    if newLevel == 6 then
        MainValue.adsManager:AdsHelperCall("interstitialAd",2);
    end;
    
    if newLevel == 11 then
        MainValue.statisticsManager:CustomTutorialComplete();
    -- 2015-6-18 暂时注释
      MainValue.statisticsManagerOG:CustomTutorialComplete();
    end; 
    
    print("[GameStatusManager.lua] newLevelNum ：：：",newLevel)
    --已经是新的关卡 判断是否有要解锁的道具 如果有就显示解锁道具面板  2015-1-8    
    local isUnlockingProp = MainValue.dataManager.goodsManager:havePropIsUnlockingByLv(newLevel)
    if isUnlockingProp ~= nil then
--        填出解锁道具面板 2015-1-29 临时取消
--        local UnlockItemClz = require("ui/UnlockItemPanel")
--        local unlockItem = UnlockItemClz.new()
--        unlockItem:initData(isUnlockingProp)
--        unlockItem:appear();
    end

    MainValue.mxGameManager:selectMission(newLevel);
    local playGamePanel = PlayGamePanel.create(newLevel);
    playGamePanel:appear();
end

return GameStatusManager;