require("game/model/MissionStatus.lua")

local MissionBase = class("MissionBase")


function MissionBase:initMission(items)

    self.m_maxRow = table.maxn(items);
    self.m_maxCol = table.maxn(items[1]);
    self.m_missionMatrix = items;
    self.m_referMatrix = {};
    self.m_statusMatrix = {};
    
    for roundY = 1,self.m_maxRow do 
        self.m_referMatrix[roundY] = {};
        self.m_statusMatrix[roundY] = {};
        for roundX = 1,self.m_maxCol do
            self.m_referMatrix[roundY][roundX] = 0;
            self.m_statusMatrix[roundY][roundX] = 0;
        end;
    end;

end

function MissionBase:traceModel(vectorType)

  
    print("---------------------- ----------------- --------------- ");
    
    if vectorType == 1 then
    
        for roundY = 1,self.m_maxRow do 
            local str = "";
            for roundX = 1,self.m_maxCol do
                str = str.."  "..self.m_missionMatrix[roundY][roundX]
            end
            print(str)
        end
    elseif vectorType == 2 then
         for roundY = 1,self.m_maxRow do 
            local str = "";
            for roundX = 1,self.m_maxCol do
                str = str.."  "..self.m_statusMatrix[roundY][roundX]
            end
            print(str)
        end
    elseif vectorType == 3 then
         for roundY = 1,self.m_maxRow do 
            local str = "";
            for roundX = 1,self.m_maxCol do
                str = str.."  "..self.m_referMatrix[roundY][roundX]
            end
            print(str)
        end
    end;
    print("---------------------- ----------------- --------------- ");
end


function MissionBase:setMissionMatrix(xRound, yRound,value)

    if xRound >= 1 and xRound <= self.m_maxCol and yRound >= 1 and yRound <= self.m_maxRow then
        self.m_missionMatrix[yRound][xRound] = value;
    end
end


function MissionBase:getMissionMatrix(xRound,yRound)
    if xRound == nil or yRound == nil then
    
    end
    if xRound >= 1 and xRound <= self.m_maxCol and yRound >= 1 and yRound <= self.m_maxRow then
        return self.m_missionMatrix[yRound][xRound];
    else
        return MissionGrid.BLANK;
    end
end



function MissionBase:getStatusMatrix(xRound, yRound)

    if xRound >= 1 and xRound <= self.m_maxCol and yRound >= 1 and yRound <= self.m_maxRow then
    
        return self.m_statusMatrix[yRound][xRound];
    else
        return MissionStatus.CLOSED;
    end
end

function MissionBase:getStatusMatrixByCoordina(coordina)

    return self:getStatusMatrix(coordina.x, coordina.y);
end

function MissionBase:setStatusMatrix( xRound,  yRound,  status)

    if xRound >= 1 and xRound <= self.m_maxCol and yRound >= 1 and yRound <= self.m_maxRow then
    
        self.m_statusMatrix[yRound][xRound] = status;
    end
end




function MissionBase:setReferMatrix( xRound,  yRound,  type)

    if xRound >= 1 and xRound <= self.m_maxCol and yRound >= 1 and yRound <= self.m_maxRow then
    
       self.m_referMatrix[yRound][xRound] = type;
    end
end

function MissionBase:getReferMatrix( xRound,  yRound)
    if xRound >= 1 and xRound <= self.m_maxCol and yRound >= 1 and yRound <= self.m_maxRow then 
    
        return self.m_referMatrix[yRound][xRound];
    
    else
    
        return 0;
    end
end

function MissionBase:getMaxRow()

    return self.m_maxRow;
end

function MissionBase:getMaxCol()

    return self.m_maxCol;
end




function MissionBase:initBrick(bricks)

    self.m_brickMatrix = bricks;
end



function MissionBase:setBrickMatrix( roundX,  roundY,  value)

    if roundX >= 1 and roundX <= self.m_maxCol and roundY >= 1 and roundY <= self.m_maxRow then
    
        self.m_brickMatrix[roundY][roundX] = value;
    end
end



function MissionBase:getBrickMatrix( roundX,  roundY)

    if roundX >= 1 and roundX <= self.m_maxCol and roundY >= 1 and roundY <= self.m_maxRow then 
    
        return self.m_brickMatrix[roundY][roundX];
    end
    return -10;
end

function MissionBase:getBrickList()
    local tempList = {};
    for roundY = 1,self.m_maxRow do 
        tempList[roundY] = {};
      
        for roundX = 1,self.m_maxCol do
        
            tempList[roundY][roundX] = 0;
            
            if self.m_brickMatrix[roundY][roundX] >= 0 then
            
                tempList[roundY][roundX] = 1;
            end
        end;
    end;

    return tempList;
end

function MissionBase:getTopBrickList()
    local tempList = {};

    for roundY = 1,self.m_maxRow do 
        tempList[roundY] = {};
      
        for roundX = 1,self.m_maxCol do
        
            tempList[roundY][roundX] = 0;
            
            if self.m_brickMatrix[roundY][roundX] ~= -2 then
            
                tempList[roundY][roundX] = 1;
            end
        end;
    end;
    return tempList;
end


return MissionBase
