require "game.view.extensions.CocosUtil"
local BackBoxBorder = class("BackBoxBorder",function()
    return cc.Node:create();
end);

local DIR = 
{
    DIR_1 = 1,
    DIR_2 = 2,
    DIR_3 = 3,
    DIR_4 = 4,
}


function BackBoxBorder:ctor()
    --self.readPermissions = ;
    
    --Vector<cocos2d::SpriteFrame*> m_edgeVector;
    --Vector<cocos2d::SpriteFrame*> m_outsideAngleVector;
    --Vector<cocos2d::SpriteFrame*> m_innerAngleVector;
    --Vector<cocos2d::SpriteFrame*> m_cellsVector;
    --std::vector< std::vector<int> > m_angleVector;
    --const int _spanX;
    --const int _spanY;
    --const float _halfSpanX;
    --const float _halfSpanY;
    --const int _cellW;
    --const int _cellH;
    --const bool _isTop;
    --int _gridW;
    --int _gridH;
    --int _totleH;
    
    self.m_edgeVector = {};
    self.m_outsideAngleVector = {};
    self.m_innerAngleVector = {};
    self.m_cellsVector = {};
    self.m_angleVector = {};
    self._spanX = 0;
    self._spanY = 0;
    self._halfSpanX = 0;
    self._halfSpanY = 0;
    self._cellW = 0;
    self._cellH = 0;
    self._isTop = false;
    self._gridW = 0;
    self._gridH = 0;
    self._totleH = 0;
end

function BackBoxBorder.create( outS,  inS,  bg1S,  bg2S, isTop,isNoBorder)
    local temp = BackBoxBorder.new();
    
    local outC = cc.SpriteFrameCache:getInstance():getSpriteFrame(outS);
    local inC  = cc.SpriteFrameCache:getInstance():getSpriteFrame(inS);
    local bg1  = cc.SpriteFrameCache:getInstance():getSpriteFrame(bg1S);
    local bg2  = cc.SpriteFrameCache:getInstance():getSpriteFrame(bg2S);
    
    if outC == nil or inC == nil or bg1 == nil or bg2 == nil then
        return nil;
    end
    local tt = cc.rect(0,0,0,0);
    
    local borderW = outC:getRect().width;
    local borderH = outC:getRect().height;
    local cellW = bg1:getRect().width;
    local cellH = bg1:getRect().height;
   
    temp._spanX = ( borderW - cellW );
    temp._spanY = ( borderH - cellH );
    temp._halfSpanX = ( ( borderW - cellW ) / 2 );
    temp._halfSpanY = ( ( borderH - cellH ) / 2 );
    temp._cellW = ( cellW );
    temp._cellH = ( cellH );
    temp._isTop = ( isTop );
    temp._isNoBorder = isNoBorder;
    temp._gridW = ( 0 );
    temp._gridH = ( 0 );
    temp._totleH = ( 0 );
    
    temp:init(outC, inC, bg1, bg2);
    
    return temp;
end

function BackBoxBorder:init(outC, inC, bg1, bg2)
    if #self.m_outsideAngleVector > 0 then 
        return nil;
    end
    --CocosUtil::extractGrid9( out->getTexture() , out->getRect() , Rect( _spanX , _spanY , _cellW - _spanX , _cellH - _spanY ) );
    --CocosUtil::extractGrid9( in->getTexture() , in->getRect() , Rect( _spanX , _spanY , _cellW - _spanX , _cellH - _spanY ) );
    
    local outFrameList = CocosUtil:extractGrid9(outC:getTexture() , outC:getRect() , cc.rect( self._spanX , self._spanY , self._cellW - self._spanX , self._cellH - self._spanY ) );
    local inFrameList = CocosUtil:extractGrid9(inC:getTexture() , inC:getRect() , cc.rect( self._spanX , self._spanY , self._cellW - self._spanX , self._cellH - self._spanY ) );

    table.insert(self.m_outsideAngleVector,outFrameList[1]);
    table.insert(self.m_outsideAngleVector,outFrameList[3]);
    table.insert(self.m_outsideAngleVector,outFrameList[9]);
    table.insert(self.m_outsideAngleVector,outFrameList[7]);
    
    table.insert(self.m_innerAngleVector,inFrameList[1]);
    table.insert(self.m_innerAngleVector,inFrameList[3]);
    table.insert(self.m_innerAngleVector,inFrameList[9]);
    table.insert(self.m_innerAngleVector,inFrameList[7]);

    table.insert(self.m_edgeVector, outFrameList[2]);
    table.insert(self.m_edgeVector, outFrameList[6]);
    table.insert(self.m_edgeVector, outFrameList[8]);
    table.insert(self.m_edgeVector, outFrameList[4]);

    table.insert(self.m_cellsVector, bg1);
    table.insert(self.m_cellsVector, bg2);
    
    for outI = 1,#self.m_outsideAngleVector do
        self.m_outsideAngleVector[outI]:retain()
    end
    
    for outI = 1,#self.m_innerAngleVector do
        self.m_innerAngleVector[outI]:retain()
    end
    for outI = 1,#self.m_edgeVector do
        self.m_edgeVector[outI]:retain()
    end
    for outI = 1,#self.m_cellsVector do
        self.m_cellsVector[outI]:retain()
    end
    
    
end

function BackBoxBorder:refresh(tarmap)
    if (#self.m_angleVector > 1)then
        self.m_angleVector = {};
    end

    self._gridH = #tarmap;
    self._gridW = #tarmap[1];

    self._totleH = self._gridH * self._cellW + self._spanY;

    if  not self._isTop then
        self:drawCellsBackground( tarmap );
    end
    
    for roundY = 1,self._gridH do
       for roundX = 1,self._gridW do
            if tarmap[roundY][roundX] > 0 then
                self:drawBorder( tarmap , roundY , roundX);
            end
        end
    end
    if self._isTop == true then 
        self:drawCellsBackground( tarmap );
    end
end

function BackBoxBorder:clear()
    self:removeAllChildren();
end

function BackBoxBorder:drawCellsBackground(tarmap)
    for roundY = 1, self._gridH do
        for roundX = 1,self._gridW do
            if tarmap[roundY][roundX] ~= 0 then
                local tempBitmap = cc.Sprite:createWithSpriteFrame(self.m_cellsVector[( roundX + roundY ) % 2 + 1]);
                tempBitmap:setAnchorPoint(0,1);                
                self:setPst( tempBitmap , (roundX - 1) * self._cellW +  self._halfSpanX , (roundY - 1) *  self._cellH +  self._halfSpanY );
                self:addChild( tempBitmap , 0 );
            end
        end
    end
end

function BackBoxBorder:drawBorder( tarmap, roundY, roundX)
    --is empty
    if self._isNoBorder == true then return end;
    local up = ( roundY == 1 or tarmap[roundY - 1][roundX] == 0 );
    local right = ( roundX == self._gridW or tarmap[roundY][roundX + 1] == 0 );
    local down = ( roundY == self._gridH or tarmap[roundY + 1][roundX] == 0 );
    local lift = ( roundX == 1 or tarmap[roundY][roundX - 1] == 0 );

    local lift_up = ( roundY == 1 or roundX == 1 or tarmap[roundY - 1][roundX - 1] == 0 );
    local right_up = ( roundY == 1 or roundX == self._gridW or tarmap[roundY - 1][roundX + 1] == 0 );
    local right_down = ( roundY == self._gridH or roundX == self._gridW or tarmap[roundY + 1][roundX + 1] == 0 );
    local lift_down = ( roundY == self._gridH or roundX == 1 or tarmap[roundY + 1][roundX - 1] == 0 );

--    cclog(" x = "..roundX.." y = "..roundY);
--    cclog(" up:%s, right:%s, down:%s, lift:%s ",tostring(up),tostring(right),tostring(down),tostring(lift));
--    cclog(" lift_up:%s, right_up:%s, right_up:%s, right_down:%s ",tostring(lift_up),tostring(right_up),tostring(right_down),tostring(lift_down));
    
    local groupId = 0;
    if  up then    
        if ((not lift_up or lift ) and (not right_up or right )) then 
            self:drawEdgeLine( DIR.DIR_1 , roundY , roundX , 1 );
        elseif (not lift_up or lift )
            then self:drawEdgeLine( DIR.DIR_1 , roundY , roundX , 2 );
        elseif( not right_up or right )
            then self:drawEdgeLine( DIR.DIR_1 , roundY , roundX , 3 );
        else 
            self:drawEdgeLine( DIR.DIR_1 , roundY , roundX , 4 );
        end
    end
    if right then
        if( ( not right_up or up ) and ( not right_down or down ) ) then groupId = 1;
        elseif( not right_up or up ) then groupId = 2;
        elseif( not right_down or down ) then groupId = 3;
        else groupId = 4;end
        self:drawEdgeLine( DIR.DIR_2 , roundY , roundX , groupId );
    end
    if down then
        if( ( not lift_down or lift ) and ( not right_down or right ) ) then self:drawEdgeLine( DIR.DIR_3 , roundY , roundX , 1 );
        elseif( not lift_down or lift )then self:drawEdgeLine( DIR.DIR_3 , roundY , roundX , 2 );
        elseif( not right_down or right )then self:drawEdgeLine( DIR.DIR_3 , roundY , roundX , 3 );
        else self:drawEdgeLine( DIR.DIR_3 , roundY , roundX , 4 );end
    end
    if lift then
        if( ( not lift_up or up ) and ( not lift_down or down ) ) then groupId = 1;
        elseif( not lift_up or up ) then groupId = 2;
        elseif( not lift_down or down )then  groupId = 3;
        else groupId = 4;end
        self:drawEdgeLine( DIR.DIR_4 , roundY , roundX , groupId );
    end

    if( lift_up and up and lift )then self:drawOutsideAngle( DIR.DIR_1 , roundY , roundX );end
    if( right_up and up and right )then self:drawOutsideAngle( DIR.DIR_2 , roundY , roundX );end
    if( right_down and down and right )then self:drawOutsideAngle( DIR.DIR_3 , roundY , roundX );end    
    if( lift_down and down and lift )then self:drawOutsideAngle( DIR.DIR_4 , roundY , roundX );end

    if( not right and not down and right_down )then self:drawInnerAngle( DIR.DIR_1 , roundY , roundX );end
    if( not lift and not down and lift_down )then self:drawInnerAngle( DIR.DIR_2 , roundY , roundX );end
    if( not lift and not up and lift_up ) then self:drawInnerAngle( DIR.DIR_3 , roundY , roundX );end
    if( not right and not up and right_up )then self:drawInnerAngle( DIR.DIR_4 , roundY , roundX );end

    if( right and down and not right_down ) then
        self:drawInnerAngle( DIR.DIR_4 , roundY + 1 , roundX , true );
        self:drawInnerAngle( DIR.DIR_2 , roundY , roundX + 1 , true );
    end

    if( lift and down and not lift_down )then
        self:drawInnerAngle( DIR.DIR_1 , roundY , roundX - 1 , true );
        self:drawInnerAngle( DIR.DIR_3 , roundY + 1 , roundX , true );
    end
end

function BackBoxBorder:drawEdgeLine( dir , roundY , roundX , group )
    local tempBitmap = cc.Sprite:createWithSpriteFrame(self.m_edgeVector[dir]);
    

    local tempBitmapX = (roundX - 1) * self._cellW;
    local tempBitmapY = (roundY - 1) * self._cellH;


    local tempW = 0; 
    local tempH = 0; 

    local tempScaleX = 0;
    local tempScaleY = 0;

    if( dir == DIR.DIR_1 or dir == DIR.DIR_3 ) then
        tempW = self._cellW - self._spanX;
        tempH = self._spanY;
        tempBitmapX = tempBitmapX + self._halfSpanX;
        if( dir == DIR.DIR_3 ) then
            tempBitmapY = tempBitmapY + self._cellH;
        end
        if( group == 1 ) then
            tempScaleX = ( self._cellW - self._spanX ) / tempW;
            tempScaleY = 1;
            tempBitmapX = tempBitmapX + self._halfSpanX;
        elseif( group == 2 ) then
            tempScaleX = ( self._cellW - self._halfSpanX ) / tempW;
            tempScaleY = 1;
            tempBitmapX = tempBitmapX + self._halfSpanX;
        elseif( group == 3 ) then 
            tempScaleX = ( self._cellW - self._halfSpanX ) / tempW;
            tempScaleY = 1;
        elseif( group == 4 ) then 
            tempScaleX = self._cellW / tempW;
            tempScaleY = 1;
        end
    elseif( dir == DIR.DIR_2 or dir == DIR.DIR_4 ) then
        tempW = self._spanX;
        tempH = self._cellH - self._spanY;

        tempBitmapY = tempBitmapY + self._halfSpanY;
        if( dir == DIR.DIR_2 ) then tempBitmapX = tempBitmapX + self._cellW;end
        if( group == 1 )then 
            tempScaleX = 1;
            tempScaleY = ( self._cellH - self._spanY ) / tempH;
            tempBitmapY = tempBitmapY + self._halfSpanY;
        elseif( group == 2 ) then
            tempScaleX = 1;
            tempScaleY = ( self._cellH - self._halfSpanY ) / tempH;
            tempBitmapY = tempBitmapY + self._halfSpanY;
        elseif( group == 3 )then
            tempScaleX = 1;
            tempScaleY = ( self._cellH - self._halfSpanY ) / tempH;
        elseif( group == 4 )then
            tempScaleX = 1;
            tempScaleY = self._cellH / tempH;
        end
    end
    tempBitmap:setAnchorPoint( 0 , 1 );
    self:addChild( tempBitmap , 0 );
    self:setPst( tempBitmap , tempBitmapX , tempBitmapY );

    tempBitmap:setScale( tempScaleX, tempScaleY );
end

function BackBoxBorder:drawOutsideAngle( dir , roundY , roundX )
    local tempBitmap =cc.Sprite:createWithSpriteFrame(self.m_outsideAngleVector[dir]);

    local tempBitmapX = (roundX - 1) * self._cellW;
    local tempBitmapY = (roundY - 1) * self._cellH;
    if( dir == DIR.DIR_1 ) then
        tempBitmapX = tempBitmapX + 0;
        tempBitmapY = tempBitmapY + 0;
    elseif( dir == DIR.DIR_2 ) then
        tempBitmapX = tempBitmapX + self._cellW;
        tempBitmapY = tempBitmapY + 0;
    elseif( dir == DIR.DIR_3 ) then
        tempBitmapX = tempBitmapX + self._cellW;
        tempBitmapY = tempBitmapY + self._cellH;
    elseif( dir == DIR.DIR_4 ) then
        tempBitmapX = tempBitmapX + 0;
        tempBitmapY = tempBitmapY + self._cellH;
    end
    tempBitmap:setAnchorPoint( 0, 1);
    self:setPst( tempBitmap , tempBitmapX , tempBitmapY );
    self:addChild( tempBitmap , 0 );
end

function BackBoxBorder:drawInnerAngle( dir , roundY , roundX , merge )
    local tempBitmap =cc.Sprite:createWithSpriteFrame(self.m_innerAngleVector[dir]);

    local tempBitmapX = (roundX - 1) * self._cellW;
    local tempBitmapY = (roundY - 1) * self._cellH;
    if( dir == DIR.DIR_1 ) then
        tempBitmapX = tempBitmapX + self._cellW;
        tempBitmapY = tempBitmapY + self._cellH;
    elseif( dir == DIR.DIR_2 )then
        tempBitmapX = tempBitmapX + 0;
        tempBitmapY = tempBitmapY + self._cellH;
    elseif( dir == DIR.DIR_3 )then
        tempBitmapX = tempBitmapX + 0;
        tempBitmapY = tempBitmapY + 0;
    elseif( dir == DIR.DIR_4 )then
        tempBitmapX = tempBitmapX + self._cellW;
        tempBitmapY = tempBitmapY + 0;
    end
    tempBitmap:setAnchorPoint( 0 , 1);
    self:setPst( tempBitmap , tempBitmapX , tempBitmapY );
    self:addChild( tempBitmap , 0 );
end

function BackBoxBorder:setPst( tempCCSprite , xPst , yPst )
    tempCCSprite:setPosition( xPst , self._totleH - yPst );
end

function BackBoxBorder:getOffsetX()
    return self._halfSpanX;
end

function BackBoxBorder:getOffsetY()
    return self._halfSpanY;
end

return BackBoxBorder;

--    local up = ( roundY == 0 or tarmap[roundY - 1][roundX] == 0 );
--    local right = ( roundX + 1 == self._gridW or tarmap[roundY][roundX + 1] == 0 );
--    local down = ( roundY + 1 == self._gridH or tarmap[roundY + 1][roundX] == 0 );
--    local lift = ( roundX == 0 or tarmap[roundY][roundX - 1] == 0 );
--
--    local lift_up = ( roundY == 0 or roundX == 0 or tarmap[roundY - 1][roundX - 1] == 0 );
--    local right_up = ( roundY == 0 or roundX + 1 == self._gridW or tarmap[roundY - 1][roundX + 1] == 0 );
--    local right_down = ( roundY + 1 == self._gridH or roundX + 1 == self._gridW or tarmap[roundY + 1][roundX + 1] == 0 );
--    local lift_down = ( roundY + 1 == self._gridH or roundX == 0 or tarmap[roundY + 1][roundX - 1] == 0 );

--    if  up then    
--        if ((lift_up == false or lift ) and ( right_up == false or right )) then 
--            self:drawEdgeLine( DIR.DIR_1 , roundY , roundX , 1 );
--        elseif (lift_up == false or lift )
--        then self:drawEdgeLine( DIR.DIR_1 , roundY , roundX , 2 );
--        elseif( right_up == false or right )
--        then self:drawEdgeLine( DIR.DIR_1 , roundY , roundX , 3 );
--        else 
--            self:drawEdgeLine( DIR.DIR_1 , roundY , roundX , 4 );
--        end
--    end
--    if right then
--        if( ( right_up == false or up ) and ( right_down == false or down ) ) then groupId = 1;
--        elseif( right_up == false or up ) then groupId = 2;
--        elseif( right_down == false or down ) then groupId = 3;
--        else groupId = 4;end
--        self:drawEdgeLine( DIR.DIR_2 , roundY , roundX , groupId );
--    end
--    if down then
--        if( ( lift_down == false or lift ) and ( right_down == false or right ) ) then self:drawEdgeLine( DIR.DIR_3 , roundY , roundX , 1 );
--        elseif( lift_down == false or lift )then self:drawEdgeLine( DIR.DIR_3 , roundY , roundX , 2 );
--        elseif( right_down == false or right )then self:drawEdgeLine( DIR.DIR_3 , roundY , roundX , 3 );
--        else self:drawEdgeLine( DIR.DIR_3 , roundY , roundX , 4 );end
--    end
--    if lift then
--        if( ( lift_up == false or up ) and ( lift_down == false or down ) ) then groupId = 1;
--        elseif( lift_up == false or up ) then groupId = 2;
--        elseif( lift_down == false or down )then  groupId = 3;
--        else groupId = 4;end
--        self:drawEdgeLine( DIR.DIR_4 , roundY , roundX , groupId );
--    end
--
--    if( lift_up and up and lift )then self:drawOutsideAngle( DIR.DIR_1 , roundY , roundX );end
--    if( right_up and up and right )then self:drawOutsideAngle( DIR.DIR_2 , roundY , roundX );end
--    if( right_down and down and right )then self:drawOutsideAngle( DIR.DIR_3 , roundY , roundX );end    
--    if( lift_down and down and lift )then self:drawOutsideAngle( DIR.DIR_4 , roundY , roundX );end