local SettingBar = class("SettingBar",function()
    return ccs.GUIReader:getInstance():widgetFromJsonFile("ui/SettingBar.json");
end)

function SettingBar:ctor()
    self.isInGame = false;
    
    self.musicBtn = nil;
    self.effectBtn = nil;
    self.exitBtn = nil;
    self.setFBBtn = nil;
    
    self.musicImag = nil;
    self.effectImag = nil;
    self.settingBtn = nil;
end

function SettingBar.create()
    local vSize = cc.Director:getInstance():getVisibleSize()
    local vOrigin = cc.Director:getInstance():getVisibleOrigin()
    
    local layer = SettingBar.new()
    layer:setPosition(vOrigin.x,vOrigin.y)
    
    layer:init();
    
    return layer
end

function SettingBar:init()

    local conPanel = self:getChildByName("ConPanel")
    conPanel:setContentSize(61,272)
    conPanel:setPositionY(-conPanel:getContentSize().height)
    conPanel:setVisible(false)
    conPanel:setOpacity(0)

    self.musicBtn = conPanel:getChildByName("MusicBtnE");
    self.effectBtn = conPanel:getChildByName("EffectBtnE");
    self.exitBtn = conPanel:getChildByName("ExitBtn");
    self.FBSetBtn = conPanel:getChildByName("FBSettingBtn");

    self.musicImag = self.musicBtn:getChildByName("DisImg");
    self.effectImag = self.effectBtn:getChildByName("DisImg");

    self.settingBtn = tolua.cast(self:getChildByName("SettingBtn"),"ccui.Button")
    local function onSetting(sender, eventType)
        if eventType == ccui.TouchEventType.ended then
            self:show(not conPanel:isVisible())
            
            cc.Director:getInstance():setDisplayStats(not cc.Director:getInstance():isDisplayStats())
        end    
    end
    self.settingBtn:addTouchEventListener(onSetting)

    local function onMusic(sender, eventType)
        if eventType == ccui.TouchEventType.ended then
            local isPlay = MainValue.dataManager.expandManager:getBgMusice();
            self:updataMusic(isPlay);
        end    
    end

    local function onEffect(sender, eventType)
        if eventType == ccui.TouchEventType.ended then
            local isPlay = MainValue.dataManager.expandManager:getEfMusice();
            self:updataEffect(isPlay);
        end    
    end
    
    local function onClose(sender, eventType)
        if eventType == ccui.TouchEventType.ended then
            MainStatus:onExitMap();
            MainStatus:onEnterSplash();
        end    
    end
    
    local function onSet(sender, eventType)
        if eventType == ccui.TouchEventType.ended then
            local SettingPanelCls = require("ui.setting.SettingPanel");
            local settingPanel = SettingPanelCls.create();
            settingPanel:appearAlone();
        end    
    end
    
    self.musicBtn:addTouchEventListener(onMusic);
    self.effectBtn:addTouchEventListener(onEffect);
    self.exitBtn:addTouchEventListener(onClose);
    self.FBSetBtn:addTouchEventListener(onSet);
end

function SettingBar:show(b_show)
    self:setTouchEnabled(false)
    local conPanel = self:getChildByName("ConPanel")
    local size = conPanel:getContentSize()
    local seq = nil
    if self.isInGame == true then
        if MainValue.mxGameManager:getCanClickGameUI() == false then
            return
        end
        local inGameExitPopupClz = require("ui/inGame/InGameExitPanel")
        local inGameExitPopup = inGameExitPopupClz.create();
        inGameExitPopup:appear();
        return
    end
    if b_show then
        conPanel:setVisible(true)
        self:initMusic(MainValue.dataManager.expandManager:getBgMusice());
        self:initEffect(MainValue.dataManager.expandManager:getEfMusice());
        seq = cc.Sequence:create(cc.EaseBackOut:create(cc.MoveTo:create(0.3, cc.p(0, 0))),
            cc.CallFunc:create(function()print("show")end))
    else
        seq = cc.Sequence:create(cc.MoveTo:create(0.3, cc.p(0, -conPanel:getContentSize().height)),
            cc.CallFunc:create(function() 
                conPanel:setVisible(false)
            end))
    end 
    conPanel:runAction(seq)   
end 

function SettingBar:initMusic(isPlaying)
    if isPlaying == nil or isPlaying == true  then
        self.musicImag:setVisible(false)
   --     MainValue.soundManager:playMusic(0,true);

    else
        self.musicImag:setVisible(true);
   --     MainValue.soundManager:stopMusic();
    end
end

function SettingBar:initEffect(isPlaying)
    if isPlaying == nil or isPlaying == true then
        self.effectImag:setVisible(false)
    else
        self.effectImag:setVisible(true);
    end
end

function SettingBar:updataMusic(isPlaying)
   
    if isPlaying == nil then
        self.musicImag:setVisible(true);
        MainValue.dataManager.expandManager:setBgMusice(false);
        MainValue.soundManager:stopMusic();
        return;
    end

    if isPlaying == true then
        self.musicImag:setVisible(true)
        MainValue.dataManager.expandManager:setBgMusice(false);
        MainValue.soundManager:stopMusic();

    else
        self.musicImag:setVisible(false);
        MainValue.dataManager.expandManager:setBgMusice(true);
        MainValue.soundManager:playMusic(0,true);


    end
    
end

function SettingBar:updataEffect(isPlaying)

    if isPlaying == nil then
        self.effectImag:setVisible(true)
        MainValue.dataManager.expandManager:setEfMusice(false);
        return;
    end
    if isPlaying == true then
        self.effectImag:setVisible(true)
        MainValue.dataManager.expandManager:setEfMusice(false);
    else
        self.effectImag:setVisible(false);
        MainValue.dataManager.expandManager:setEfMusice(true);

    end
end

function SettingBar:isIngame(isIngame)
    self.isInGame = isIngame;
    if self.isInGame == true then
       local conPanel = self:getChildByName("ConPanel")
       conPanel:setVisible(false)
    end
   
end

return SettingBar