local InboxPortal = class("InboxPortal", function()
    return cc.Node:create();
end)

function InboxPortal:ctor()
    self.inboxIcon = nil;
    self.numBg = nil;
    self.numLab = nil;
    
    self.timerId = nil;
    
    self.REFRESH_TIME = 600;
end

function InboxPortal.create()
    local temp = InboxPortal.new();
    temp:init()
    return temp;
end

function InboxPortal:init()
    self.inboxIcon = ccui.Button:create("mapIcon/iconInbox.png","mapIcon/iconInbox.png", "", 0)
    self:addChild(self.inboxIcon);
    
    self.numLab = FacFont:getFontsBmp(FontType.FontWhite, "Oz")
    self.numLab:setScale(0.4);
    self.numLab:setPosition(-1, -1);
    self:addChild(self.numLab);
    
    local function clickHandler(sender, eventType)
        if eventType == ccui.TouchEventType.ended then
            if not MainValue.mxUIManager.mapManager:getIsTouch() then
                return;
            end

            if MainValue.fbManager:getIsLogin() then --已连接，需要显示loading界面
                local function getMesSuccess(i, data)
                    MainValue.mxLoadingManager:removeLoading();
                    
                    local inboxClz = require("ui.inbox.InboxPanel")
                    
                    local inbox = inboxClz.create(data);
                    inbox:appear();
                end
                
                MainValue.fbManager:getMessage(getMesSuccess)
                         
                MainValue.mxLoadingManager:addLoading();
                
            else                                    --未连接，需连接FB，不需要显示loading界面
                local function callBack()
                    local function getMesSuccess(i, data)
                        local inboxClz = require("ui.inbox.InboxPanel")
                        local inbox = inboxClz.create(data);
                        inbox:appear();
                    end
                    MainValue.fbManager:getMessage(getMesSuccess)
                end
                
                MainValue.fbManager:isLoginByCallback(callBack);
            end
        end
    end
    self.inboxIcon:addTouchEventListener(clickHandler);    
    self:needUpdataNum();
end

--[[是否需要更新数量]]
function InboxPortal:needUpdataNum()
    if MainValue.fbManager:getIsLogin() then
        self:updataMesNum();
        
        local delayTime = 0;
        local function update(dt)
            delayTime =  delayTime + dt;

            if delayTime > self.REFRESH_TIME then
                delayTime = 0;

                self:updataMesNum();
            end
        end
        if self.timerId == nil then
            self.timerId = cc.Director:getInstance():getScheduler():scheduleScriptFunc(update, 0, false)
        end
    else
        if self.timerId ~= nil then
            cc.Director:getInstance():getScheduler():unscheduleScriptEntry(self.timerId)
        end
        
        if self.numLab ~= nil then
            self.numLab:setText("Oz");
        end
    end
end

--[[更新消息数量]]--
function InboxPortal:updataMesNum()
    local function getMesSuccess(i, data)
        if data == nil then 
            return;
        end
        
        local mesCount = table.getn(data);
        local countTxt = nil;
        if mesCount > 0 then
            countTxt = tostring(mesCount);
        else
            countTxt = "Oz";
        end
        
        if self.numLab ~= nil then
            self.numLab:setText("Oz");
        end
        
--        if mesCount > 0 then
--            if self.numBg == nil then
--                self.numBg = cc.Sprite:createWithSpriteFrameName("ui/ui/bgMesNum.png")
--                self.numBg:setPosition(25,20);
--                self:addChild(self.numBg);
--            end
--
--            if self.numLab ~= nil then
--                self.numLab:removeFromParent(true);
--            end
--
--            self.numLab = FacFont:getFontsBmp(FontType.FontWhite, tostring(mesCount))
--            self.numLab:setScale(0.4);
--            self.numLab:setPosition(14,17);
--            self.numBg:addChild(self.numLab);
--            
--            self.numLab:setVisible(true);
--            self.numBg:setVisible(true);
--        else
--            if self.numLab ~= nil then
--                self.numLab:setVisible(false);
--            end
--
--            if self.numBg ~= nil then
--                self.numBg:setVisible(false);
--            end
--        end
    end
    MainValue.fbManager:getMessage(getMesSuccess)
end


return InboxPortal;
