PanelManager = class("PanelManager",function()    
    return cc.Layer:create();
end);

PanelAppearType = {
    COME_DOWN = 11;
    COME_UP = 12;
    COME_LEFT = 13;
    COME_RIGHT = 14;
    
    FADE_IN = 21;
    
    POP_UP = 31;
    
    NONE = 91;
}

PanelDisappearType = {
    LEAVE_DOWN = 111;
    LEAVE_UP = 112;
    LEAVE_LEFT = 113;
    LEAVE_RIGHT = 114;
    
    FADE_OUT = 121;
    
    POP_UNDER = 131;
    
    NONE = 191;
}

function PanelManager:ctor()
    self.uiList = {};
    self.maskBg = nil;
    
    
--  显示单独面板的可以加载已经显示队列上的面板的处理 2015-1-6
    self.uiAloneList = {}
    
    self.isListShowing = false --当前队列消失面板是否显示中
   
end;


function PanelManager:create()
    local layer = PanelManager.new();
    return layer;
end;

--@uiNode
--@anmType
--@callFun 添加单独当前显示面板
function PanelManager:addPanelAlone(uiNode,anmType,callFun,disappearFun)
    if(self.uiAloneList == nil) then 
        self.uiAloneList = {}
    end    
    if uiNode == nil then 
        return
    end
    local showUIDataCLZ = require("ui/panelManager/ShowUIData");
    local showUIData = showUIDataCLZ.creat()
    showUIData.anmType = anmType
    showUIData.uiNode = uiNode
    showUIData.callFun = callFun
    showUIData.disappearFun = disappearFun;
    showUIData.uiNode:retain()
    table.insert(self.uiAloneList,showUIData)
    showUIData.name = tostring(table.maxn(self.uiAloneList))
    showUIData.isShowing = true
    
    self:showUIAlone(showUIData)    
end



function PanelManager:showUIAlone(showUIDataPar)

    if showUIDataPar.isShowing==false then 
        return
    end

    local mCallBackFun = function()
        if(showUIDataPar.callFun) then
            showUIDataPar.callFun(showUIDataPar.uiNode)  
        end
    end
    MainValue.soundManager:playEffect(GameSoundEffectId.OPEN_D)
    if showUIDataPar.anmType < 20 then
        MainValue.actionManager:createUIAction():UIAppear(showUIDataPar.uiNode, showUIDataPar.anmType, mCallBackFun);

    elseif showUIDataPar.anmType < 30 then
        MainValue.actionManager:createFadeAction():FadeIn(showUIDataPar.uiNode,mCallBackFun);
        
    elseif showUIDataPar.anmType < 40 then
        MainValue.actionManager:createPopAction():PopUp(showUIDataPar.uiNode,mCallBackFun);
          
    elseif showUIDataPar.anmType == PanelAppearType.NONE then 
        mCallBackFun();
    end;
    
    local aloneLen = #self.uiAloneList;
    
    self:resetMaskBg(false,aloneLen)
    self:addChild(showUIDataPar.uiNode,aloneLen + 1)       
    
    self:dispatchPanelOpen(); 
end

function PanelManager:removePanelAlone(tarNodePar,closeAnmType,callBackFun)

    local showNodeAloneIndex = 0
    local tarShowingPanelAlone = nil
    local showingPanel = nil
    for nodeI = 1 , #(self.uiAloneList) do
        if self.uiAloneList[nodeI].uiNode == tarNodePar then
            showNodeAloneIndex = nodeI
            tarShowingPanelAlone = self.uiAloneList[showNodeAloneIndex].uiNode
            showingPanel = tarNodePar
        end    
    end    
    
    local mCallBackFun = function()
        table.remove(self.uiAloneList,showNodeAloneIndex)    
        tarShowingPanelAlone:release()
        tarShowingPanelAlone:removeFromParent(true)
        
        local aloneLen = #self.uiAloneList;
        if aloneLen == 0  and  not self.isListShowing then
            self:resetMaskBg(true)
        else
            self:resetMaskBg(false,aloneLen - 1)
        end
        
         if(callBackFun) then
            callBackFun(showingPanel) 
        end
        
        self:dispatchPanelClose();
    end
    MainValue.soundManager:playEffect(GameSoundEffectId.CLOSE_D);
    if closeAnmType > 110 and closeAnmType < 120 then
        MainValue.actionManager:createUIAction():UIDisappear(showingPanel, closeAnmType, mCallBackFun);

    elseif closeAnmType > 120 and closeAnmType < 130 then
        MainValue.actionManager:createFadeAction():FadeOut(showingPanel,mCallBackFun);
        
    elseif closeAnmType > 130 and closeAnmType < 140 then
        MainValue.actionManager:createPopAction():PopUnder(showingPanel,mCallBackFun);
        
    elseif closeAnmType == PanelDisappearType.NONE then 
        mCallBackFun(); 
    end;
end


--[[--
--@param uiNodePar 面板node
--@param anmType 弹出方式
--@param callFun 弹出完毕回调
--@param disappearFun 面板关闭回调（点回退键回调） by hmj
]]--
function PanelManager:addPanel(uiNodePar,anmType,callFun, disappearFun)
    if self.uiList == nil then 
        self.uiList = {}
    end
   
    if uiNodePar == nil then 
        return;
    end
    --    2015 - 1 - 7  satrt  处理快速点击出现重复队列面板
    local isExist = false 
    local listLen = table.maxn(self.uiList)
    for uiI = 1,#(self.uiList) do
        local tarNode= self.uiList[uiI].uiNode
        if tarNode.class == uiNodePar.class then
            isExist = true
        end       
    end
    if isExist then 
        return
    end    
    --    2015 - 1 - 7  end  处理快速点击出现重复队列面板
    
    
    
    local showUIDataCLZ = require("ui/panelManager/ShowUIData");
    local showUIData = showUIDataCLZ.creat();
    showUIData.anmType = anmType;
    showUIData.uiNode = uiNodePar;
    showUIData.callFun = callFun;
    showUIData.disappearFun = disappearFun;
    showUIData.uiNode:retain();    
    table.insert(self.uiList,showUIData);
    
    local aloneLen = #self.uiAloneList;
    
    local showingData = self:getShowingPanelData();
    if showingData == nil then
        showUIData.isShowing = true;
        
        if aloneLen == 0 then
            self:resetMaskBg(false,0);
        end
    else 
        showUIData.isShowing = false;
    end;
   
    self:showUI(showUIData);

end;




function PanelManager:showUI(showUIData)

    if showUIData.isShowing==false then 
        return;
    end;
    
     local mCallBackFun = function()
        if(showUIData.callFun) then
            showUIData.callFun(showUIData.uiNode);  
        end;
     end;
     
    MainValue.soundManager:playEffect(GameSoundEffectId.OPEN_D)
    if showUIData.anmType < 20 then
        MainValue.actionManager:createUIAction():UIAppear(showUIData.uiNode,showUIData.anmType,mCallBackFun);
        
    elseif showUIData.anmType < 30 then
        MainValue.actionManager:createFadeAction():FadeIn(showUIData.uiNode,mCallBackFun);
        
    elseif showUIData.anmType < 40 then
        MainValue.actionManager:createPopAction():PopUp(showUIData.uiNode,mCallBackFun);  
        
    elseif showUIData.anmType == PanelAppearType.NONE then 
        mCallBackFun(); 
    end;
    self.isListShowing = true
    self:addChild(showUIData.uiNode,0);
    
    self:dispatchPanelOpen();
end

function PanelManager:removePanel(closeAnmType,callBackFun)
    local showingPanelData = self:getShowingPanelData();
    local showingPanel = showingPanelData.uiNode    
    
    local mCallBackFun = function()
        table.remove(self.uiList,1);
        showingPanel:release();
--        showingPanel:removeFromParent(true);
        self:resetMaskBg(true);
        self.isListShowing = false
        local nextPanelData = self:getNextPanelData();
        if(nextPanelData and nextPanelData.uiNode) then
        
            nextPanelData.isShowing = true;
            self:showUI(nextPanelData);
            --self:addPanel(nextPanelData.uiNode,nextPanelData.anmType,nextPanelData.callFun)
        end;
        
        if(callBackFun) then
            callBackFun(showingPanel);  
        end;
        
        self:dispatchPanelClose();
    end 
    MainValue.soundManager:playEffect(GameSoundEffectId.CLOSE_D)
    if closeAnmType > 110 and closeAnmType < 120 then
        MainValue.actionManager:createUIAction():UIDisappear(showingPanel, closeAnmType, mCallBackFun);

    elseif closeAnmType > 120 and closeAnmType < 130 then
        MainValue.actionManager:createFadeAction():FadeOut(showingPanel,mCallBackFun);
        
    elseif closeAnmType > 130 and closeAnmType < 140 then
        MainValue.actionManager:createPopAction():PopUnder(showingPanel,mCallBackFun);
        
    elseif closeAnmType == PanelDisappearType.NONE then 
        mCallBackFun(); 
    end;
end;


function PanelManager:resetMaskBg(isRemove,deepIndex)    
    if isRemove then        
        if self.maskBg then
            self.maskBg:release()
            self.maskBg:removeFromParent(true);
            self.maskBg = nil;
         end
    else 
        if self.maskBg == nil then
            local color4B = cc.c4b(0,0,0,102);
            self.maskBg = cc.LayerColor:create(color4B,MainValue.winSize.width,MainValue.winSize.height);
            local listener = function(eventType, x, y)
               
                if eventType == "began" then
                    return self:touch_began(x, y)
                elseif eventType == "moved" then
                    self:touch_moved(x, y)
                elseif eventType == "ended" then
                    self:touch_ended(x, y)
                elseif eventType == "cancelled" then
                    self:touch_cancelled(x, y)
                end
            end
            self.maskBg:registerScriptTouchHandler(listener, false, 0, true)
            self.maskBg:setTouchEnabled(true);
            self.maskBg:retain()
        end
        if self.maskBg:getParent()~= nil then
            self.maskBg:getParent():removeChild(self.maskBg)
        end
        self:addChild(self.maskBg,deepIndex)
    end
    
end

function PanelManager:touch_began(x, y)
    return true
end
function PanelManager:touch_moved(x, y)
    return false
end
function PanelManager:touch_ended(x, y)
    return false
end
function PanelManager:touch_cancelled(x, y)
    return false
end

--[[是否有面板正在显示]]
function PanelManager:getIsPanelOnShow()
    if self:getChildrenCount() > 0 then
        return true;  
    else
        return false;
    end
end

--[[通知面板关闭]]
function PanelManager:dispatchPanelClose()
    if self:getChildrenCount() == 0 then
        if MainValue.mxUIManager.mapManager ~= nil then
            MainValue.mxUIManager.mapManager:redisplayObject();
        end
    end
end

--[[通知面板开启]]
function PanelManager:dispatchPanelOpen()
    if MainValue.mxUIManager.mapManager ~= nil then
        MainValue.mxUIManager.mapManager:hideObject();
    end
end

function PanelManager:getShowingPanelData()
    if(self.uiList == nil) then
        return nil;
     end;
     
    local length = table.maxn(self.uiList);
    if(length == 0) then
        return nil;
    end
     
    if self.uiList[1].isShowing then
        return self.uiList[1];
    end;
    
    return nil;
end;

function PanelManager:getShowingAloneData()
    if(self.uiAloneList == nil) then
        return nil;
    end;

    local length = table.maxn(self.uiAloneList);
    if(length == 0) then
        return nil;
    end

    if self.uiAloneList[1].isShowing then
        return self.uiAloneList[1];
    end;

    return nil;
end;



function PanelManager:getNextPanelData()
    if(self.uiList == nil) then
        return nil;
        end
    local length = table.maxn(self.uiList);
    if(length == 0) then
        return nil;
    end
    local nextPanel = self.uiList[1];
    return nextPanel;
end;
 

--回退键功能 
function PanelManager:backPack()
    if self:getShowingAloneData() ~= nil then
        local data = self:getShowingAloneData();
        data.disappearFun(data.uiNode);
        return true;
    end
    
    if self:getShowingPanelData() ~= nil then
        local data = self:getShowingPanelData();
        data.disappearFun(data.uiNode);
        return true;
    end

    return false;

end

return PanelManager;

