local SettingPanel = class("SettingPanel", function()
    return ccs.GUIReader:getInstance():widgetFromJsonFile("ui/SettingPanel.json");
end)

function SettingPanel:ctor()
    self.closeBtn = nil;
    self.titleLab = nil;
    self.versionLab = nil;--版本号
    
    self.musicBtn = nil;
    self.effectBtn = nil;
    
    self.musicImag = nil;
    self.effectImag = nil;
    
    self.callBack = nil;
    
    self.isIngame = false;
    
    self.isConnect = false;
    
    self.exitGame = false;
end

function SettingPanel.create(isIngame)
    local temp = SettingPanel.new()
    temp.isIngame = isIngame;
    temp:init();
    return temp
end

function SettingPanel:init()
    self:setPosition(MainValue.winSize.width / 2, MainValue.winSize.height / 2);
    self:setScale(0);
    
    local titleStr = nil;
    if self.isIngame then
        titleStr = MainValue.infosManager:getObjMultilingualByKey("Common_settingTitle_2")
    else
        titleStr = MainValue.infosManager:getObjMultilingualByKey("Common_settingTitle_1")
    end
    self.titleLab = FacFont:getFontsBmp(FontType.FontBigYellowLightShadeSide, titleStr)
    self.titleLab:setPosition(3, 240);
    self:addChild(self.titleLab);
    
    local bgPanel = self:getChildByName("PanelBg");
--    local coockyBg = SpecialGridTools.create("ui/ui/bgCooky.png",492,347,26)
--    bgPanel:addChild(coockyBg)
    
    self.closeBtn = self:getChildByName("BtnClose");
    local function onClose(sender, eventType)
        if eventType == ccui.TouchEventType.ended then
            self:disapperAlone();
        end
    end
    self.closeBtn:addTouchEventListener(onClose);
    
    self.musicBtn = self:getChildByName("BtnSound")
    self.effectBtn = self:getChildByName("BtnEffect")
    
    self.musicImag = self:getChildByName("ImgNoSound");
    self.effectImag = self:getChildByName("ImgNoEffect");
    
    self:initMusic(MainValue.dataManager.expandManager:getBgMusice());
    self:initEffect(MainValue.dataManager.expandManager:getEfMusice());
    
    local function onMusic(sender, eventType)
        if eventType == ccui.TouchEventType.ended then
            local isPlay = MainValue.dataManager.expandManager:getBgMusice();
            self:updataMusic(isPlay);
        end    
    end
    self.musicBtn:addTouchEventListener(onMusic);

    local function onEffect(sender, eventType)
        if eventType == ccui.TouchEventType.ended then
            local isPlay = MainValue.dataManager.expandManager:getEfMusice();
            self:updataEffect(isPlay);
        end    
    end
    self.effectBtn:addTouchEventListener(onEffect);
    
    if self.isIngame then
        self:initInGame();
    else 
        self:initInMap();
    end
end

--地图上的设置面板
function SettingPanel:initInMap()
    local panelInMap = self:getChildByName("PanelInMap");
    panelInMap:setVisible(true);
    
    
    local uid = MainValue.dataManager.userManager:getUID();
    self.uidLab = FacFont:getFontsBmp(FontType.FontRed, uid)
    self.uidLab:setPosition(-100, -225);
    self:addChild(self.uidLab); 
    
    local version = MainValue.platformManager:getAppVersion();
    self.versionLab = FacFont:getFontsBmp(FontType.FontRed, version)
    self.versionLab:setPosition(100, -225);
    self:addChild(self.versionLab); 
    
    local menuBtn = panelInMap:getChildByName("BtnMainMenu")
    local menuLab = FacFont:getFontsBmp(FontType.FontWhite, MainValue.infosManager:getObjMultilingualByKey("Common_settingMainMenu"))
    menuLab:setPosition(157, 58);
    menuBtn:addChild(menuLab); 

    local connectBtn = panelInMap:getChildByName("BtnConnect")
    local connectLab = FacFont:getFontsBmp(FontType.FontWhite, MainValue.infosManager:getObjMultilingualByKey("SettingPanel_Btn_Connect"))
    connectLab:setPosition(157, 58);
    connectBtn:addChild(connectLab); 

    
    local function onMainMenu(sender, eventType)
        if eventType == ccui.TouchEventType.ended then
            MainValue.actionManager:createJellyAction():run(sender,40);
            MainStatus:onExitMap();
            MainStatus:onEnterSplash();
            
            self:disapperAlone();
        end
    end
    menuBtn:addTouchEventListener(onMainMenu);

    local function onConnect(sender, eventType)
        if eventType == ccui.TouchEventType.ended then
            MainValue.actionManager:createJellyAction():run(sender,40);
            self.isConnect = true;
            self:disapperAlone();
        end
    end
    connectBtn:addTouchEventListener(onConnect);
    
    print("SettignPanel::connectBtn")
    if MainValue.fbManager:getIsLogin() then
        connectBtn:setVisible(false);
        menuBtn:setPositionY(-157);
    end
    
    ----已连接FB
    local function onSuccess()
        print("SettignPanel::onSuccess")
        connectBtn:setVisible(false);
        menuBtn:setPositionY(-157);
    end
    self.callBack = onSuccess;
    MainValue.eventDispatcher:addEventListener(MainValue.eventDispatcher.eventType.EVENT_FRIENDS, onSuccess);
end

--inGame的设置面板
function SettingPanel:initInGame()
    local panelInGame = self:getChildByName("PanelInGame");
    panelInGame:setVisible(true);
    
    local btnQuit = panelInGame:getChildByName("BtnQuit");
    local quitLab = FacFont:getFontsBmp(FontType.FontWhite, MainValue.infosManager:getObjMultilingualByKey("Common_settingQuit"))
    quitLab:setPosition(153, 59);
    btnQuit:addChild(quitLab); 
    
    local btnResume = panelInGame:getChildByName("BtnResume");
    local resumeLab = FacFont:getFontsBmp(FontType.FontWhite,MainValue.infosManager:getObjMultilingualByKey("Common_settingResume"))
    resumeLab:setPosition(155, 63);
    btnResume:addChild(resumeLab); 
    
    local function onQuit(sender,eventType)
        if eventType == ccui.TouchEventType.ended then
            --do replay
            MainValue.actionManager:createJellyAction():run(sender,40);
            MainValue.mxGameManager:gameExitGame();
            self.exitGame = true;
            
            self:disapperAlone();
        end
    end
    btnQuit:addTouchEventListener(onQuit)
    
    local function onResume(sender,eventType)
        if eventType == ccui.TouchEventType.ended then
            --do replay
            MainValue.actionManager:createJellyAction():run(sender,40);
            self:disapperAlone();
        end
    end
    btnResume:addTouchEventListener(onResume)
end

function SettingPanel:initMusic(isPlaying)
    if isPlaying == nil or isPlaying == true  then
        self.musicImag:setVisible(false)
      

    else
        self.musicImag:setVisible(true);
    
    end
end

function SettingPanel:initEffect(isPlaying)
    if isPlaying == nil or isPlaying == true then
        self.effectImag:setVisible(false)
    else
        self.effectImag:setVisible(true);
    end
end

function SettingPanel:updataMusic(isPlaying)

    if isPlaying == nil then
        self.musicImag:setVisible(true);
        MainValue.dataManager.expandManager:setBgMusice(false);
        MainValue.soundManager:stopMusic();
        return;
    end

    if isPlaying == true then
        self.musicImag:setVisible(true)
        MainValue.dataManager.expandManager:setBgMusice(false);
        MainValue.soundManager:stopMusic();

    else
        self.musicImag:setVisible(false);
        MainValue.dataManager.expandManager:setBgMusice(true);
        
        if MainValue.mxGameManager:isGameIn() then
            print("[SettingPanel]--在游戏内部")
            local mapId = math.ceil(MainValue.SELECTED_LEVEL/10);
            local musicId = mapConfig.gameMusic[mapId];
            MainValue.soundManager:playMusic(tonumber(musicId) ,true);
        else
            print("[SettingPanel]--不在在游戏内部")
            MainValue.soundManager:playMusic(0,true);
        end
    end
end

function SettingPanel:updataEffect(isPlaying)

    if isPlaying == nil then
        self.effectImag:setVisible(true)
        MainValue.dataManager.expandManager:setEfMusice(false);
        return;
    end
    if isPlaying == true then
        self.effectImag:setVisible(true)
        MainValue.dataManager.expandManager:setEfMusice(false);
    else
        self.effectImag:setVisible(false);
        MainValue.dataManager.expandManager:setEfMusice(true);
    end
end

function SettingPanel:appearAlone()
    MainValue.mxUIManager.panelManager:addPanelAlone(self, PanelAppearType.POP_UP, self.appearComplete,self.disapperAlone);
end

function SettingPanel:appearComplete()
end

function SettingPanel:disapperAlone()
    MainValue.mxUIManager.panelManager:removePanelAlone(self,PanelDisappearType.POP_UNDER , self.disapperComplete)
end

function SettingPanel:disapperComplete()
    MainValue.eventDispatcher:removeEventListener(MainValue.eventDispatcher.eventType.EVENT_FRIENDS, self.callBack);

    if self.isConnect then
        MainValue.fbManager:FBlogin();
    end
    if self.exitGame == true then
       self.exitGame = false;
       local quitGame = require("ui.exitGame.QuitGame")
       local quitGamePanel  = quitGame.create();
       quitGamePanel:setData();
       quitGamePanel:appear();
    end
    self:removeFromParent(true);
    
end

return SettingPanel