local FilesManager = class("FilesManager");
local crypto = require(cc.PACKAGE_NAME .. ".crypto");

FilesManager.CMD = 
{
    LoadingVersion = "version",
    LoadingFilelist = "filelist",
    SaveFiles = "savefiles"
}
--获取路径  
local function getFilePathByString(filename)  
    return string.match(filename, "(.+)/[^/]*%.%w+$") --*nix system  
    --return string.match(filename, “(.+)\\[^\\]*%.%w+$”) — windows  
end  

--获取文件名  
local function getFileNameByString(filename)  
    return string.match(filename, ".+/([^/]*%.%w+)$") -- *nix system  
    --return string.match(filename, “.+\\([^\\]*%.%w+)$”) — *nix system  
end 
--获取扩展名  
local function getFileExtensionByString(filename)  
    return filename:match(".+%.(%w+)$");
end

function FilesManager:ctor()
    self.host = DataConfig.updateHost .. DataConfig.os .. "_en/";
    
    self.storagePath = DataConfig.writablePath .. DataConfig.gameName .. "/";
    self.storageCachePath = DataConfig.writablePath .. DataConfig.gameName .. "_temp/";
    
    cclog("storagePath : "..self.storagePath.." storageCachePath : "..self.storageCachePath);
    
    self.liveVersion = nil;
    self.liveData = nil;
    self.localVersion = nil;
    self.localData = nil;
    self.diffData = nil;
    self.maxFiles = 0;
    self.addFiles = 0;
    
    self.deleteUpdate = false;
    self.retryConnectCount = 0;
end

function FilesManager.create(updateManager)
    local temp = FilesManager.new();
    temp.updateManager = updateManager;
    return temp;
end

--[[
--保存本地文件
--@params fileName 子目录及文件名称（并不是全路径）
--@params content 文件的内容
--需要先删正式更新目录的所有文件，然后将打包内的filelist.txt文件存到正式目录下。
]]
function FilesManager:saveToLocalFile(fileName, content)
    local fileFullPath = fileName;
    
    if fileFullPath ~= nil 
    then
        local localPath = getFilePathByString(fileFullPath).."/";
        --local localName = getFileNameByString(fileFullPath);
        
        local isCreateDirState = self:isDirectoryExist(localPath);
        
        if isCreateDirState == true
        then
            local file,error = io.open(fileFullPath,"wb");
            
            if file ~= nil 
            then
                file:write(content);
                file:close();
                
                return true;
            end--end if
        end--end if
    end--end if
    
    self.updateManager:refreshStatus(UpdateType.MAIN_STATUS.ErrorSave);
    --通知
    
    return false;
end

--[[
--读取本地文件
--@params filepath 读取文件的本地全路径(包括文件名)
]]
function FilesManager:readLocalFile(filepath)
    local content = nil;
    local file,error = io.open(filepath,"rb");

    if file ~= nil 
    then 
        content = file:read("*a");
        file:close();
    end
    
    return content, error;
end

--[[
--获取远程文件信息
--@params self 
--@params cmd 命令
--@params fileName 子目录及文件名称（并不是全路径）
]]
function FilesManager:getLiveFile(cmd, fileName)
    if fileName == nil then return ; end
    
    local xhr = cc.XMLHttpRequest:new();
    xhr.responseType = cc.XMLHTTPREQUEST_RESPONSE_STRING;
    
    xhr:open(DataConfig.updateMethod, self.host..fileName);

    local function onReadyStateChange()
        if xhr.readyState == 4 and (xhr.status >= 200 and xhr.status < 207) 
        then
            --local statusString = "Http Status Code:"..xhr.statusText
            
            print("[FilesManager] fileName:"..fileName);
            
            self.retryConnectCount = 0;
            self:onGetFilesHandler(cmd, fileName, xhr.response);
        else
            self.retryConnectCount = self.retryConnectCount + 1;
            
            cclog("[FilesManager] ERROR retryConnectCount:"..self.retryConnectCount.." readyState:".. xhr.readyState.. " status:"..xhr.status)
            
--            if self.retryConnectCount <= 10 then 
--                self:getLiveFile(cmd,fileName); 
--            else 
               self.updateManager:refreshStatus(UpdateType.MAIN_STATUS.ErrorLoadList);
--            end
        end
    end

    xhr:registerScriptHandler(onReadyStateChange);
    xhr:send();
end

function FilesManager:onGetFilesHandler(cmd, fileName, content)
    if cmd == nil 
        or fileName == nil 
        or content == nil 
        or content == "" 
    then 
        return ; 
    end

    if cmd == self.CMD.LoadingVersion
    then
        local loc_content = tostring(content);
        loc_content = lua_string_split(loc_content, " ");
        self.liveVersion = tonumber(loc_content[1]);
        self.frameworkVersion = tonumber(loc_content[2]);
        
        self.updateManager:refreshStatus(UpdateType.MAIN_STATUS.FinishedVersion);
        
    elseif cmd == self.CMD.LoadingFilelist
    then
            self.liveData = json.decode(content,1);
            self.updateManager:refreshStatus(UpdateType.MAIN_STATUS.FinishedFileList);
        
    elseif cmd == self.CMD.SaveFiles
    then
        local saveStatus = self:saveToLocalFile(self.storageCachePath..fileName,content);
        
        if saveStatus then
            self.addFiles = self.addFiles + 1;
            
            self.updateManager:loadProgress(self.addFiles,self.maxFiles);
            
            local nextFileName = self:getNextDiffFileName();
            
            if nextFileName == nil and self.addFiles == self.maxFiles
            then
                self.updateManager:refreshStatus(UpdateType.MAIN_STATUS.FinishedUpdateFiles);
                
            else
                self:getLiveFile(self.CMD.SaveFiles,nextFileName);
            end
            
        end--end if
    end

end

--将线上更新文件下载到临时目录
function FilesManager:updateDiffData(diffData, newLocalData)
    
    self.diffData = diffData;
    --self.liveVersion = newLocalVersion;
    self.liveData = {};
    self.liveData = newLocalData;
    
    local allDiff = self.diffData["all"];
    
    for key, value in pairs(allDiff) do
        self.maxFiles = self.maxFiles + 1;
    end

    local fileName = self:getNextDiffFileName();
    self:getLiveFile(self.CMD.SaveFiles,fileName)
end

function FilesManager:saveLocalFiles()
    
    local allDiff = self.diffData["all"];
    local addFiles = 0;
    
    for key, value in pairs(allDiff) do
        local fileName = key;
        local tempFullPath = self.storageCachePath..fileName;
        
        local readContent, error = self:readLocalFile(tempFullPath);
        
        if readContent ~= nil
        then
            local saveSatus = self:saveToLocalFile(self.storagePath..fileName,readContent);
            
            if saveSatus == true
            then 
                addFiles = addFiles + 1;
            end
        end--end if
    end--end for
    
    if addFiles == self.maxFiles 
    then 
        local saveVersionStatus = self:saveVersion();
        local saveFilelistStatus = self:saveFileList();
        
        if saveVersionStatus == true and saveFilelistStatus == true
        then
            self.updateManager:refreshStatus(UpdateType.MAIN_STATUS.Finished);
            self.localVersion = self.liveVersion;
            return true; 
        else
            self.updateManager:refreshStatus(UpdateType.MAIN_STATUS.ErrorSave);
            return false; 
        end
    end
end

function FilesManager:getNextDiffFileName()
    local allDiff = self.diffData["all"];
    
    for key, value in pairs(allDiff) do
        local loadStatus = value["status"];
        
        if type(loadStatus) == "nil"
        then
            self.diffData["all"][key]["status"] = 1;
            --if self.liveData == nil then end;
            return key;
        end--end if
    end--end for
    
    return nil;
end

function FilesManager:deleteDiffDataItem(fileName)

end

--获取本地更新版本号(本版本号将一直保持最新，即最新版本后会更新此版本号)
function FilesManager:getlocalVersion()
    if type(self.localVersion) == "nil" or self.localVersion == ""
    then
        return DataConfig.updateVersion;
    else 
        return self.localVersion;
    end
end

--获取本地(从机器中读取)及线上(从CDN读取)版本号
function FilesManager:getVersion()
    local fileName = "version.txt";
    
    self.localVersion = self:readLocalFile(self.storagePath..fileName);
    self.localVersion = tonumber(self.localVersion);
    
    if type(self.localVersion) == "nil" or self.localVersion == ""
    then 
        self.localVersion = DataConfig.updateVersion;
    else
        if self.localVersion < DataConfig.updateVersion
        then
            self.localVersion = DataConfig.updateVersion;
            --删除原有热更新目录
            self:disapperLocalLiveData();
        end
    end

    self:getLiveFile( self.CMD.LoadingVersion, fileName);
end

function FilesManager:saveVersion()
    local fileName = "version.txt";
    local saveStatus = self:saveToLocalFile(self.storagePath..fileName,self.liveVersion);
    return saveStatus;
end

function FilesManager:getLiveAndLocalVersion()
    return self.liveVersion, self.localVersion;
end

function FilesManager:getFrameworkVersion()
    return self.frameworkVersion;
end

function FilesManager:getFileList()
    local fileName = "filelist.txt";

    self.localData = self:readLocalFile(self.storagePath..fileName);
    
    if type(self.localData) == "string" 
    then 
        self.localData = json.decode(self.localData);
    end
    
    self:getLiveFile( self.CMD.LoadingFilelist, fileName);
end

function FilesManager:saveFileList()
    local fileName = "filelist.txt";
    
    if type(self.liveData) == "table"
    then
        self.liveData = json.encode(self.liveData);
    end
    
    if type(self.liveData) == "string"
    then
        local saveStatus = self:saveToLocalFile(self.storagePath..fileName, self.liveData);
        return saveStatus;
    end
    
    return false;
end

function FilesManager:getLiveAndLocalFileList()
    return self.liveData, self.localData;
end

function FilesManager:isDirectoryExist(filepath)
    
    local isDirectory = cc.FileUtils:getInstance():isDirectoryExist(filepath);
    
    if isDirectory == false
    then
        return cc.FileUtils:getInstance():createDirectory(filepath);
    end
    
    return isDirectory;
end

function FilesManager:packageFilelistToMd5()
    if self.liveData == nil then return; end
    if self.localData ~= nil and self.localData ~= "" then return ;end
    
    local loc_md5List = {};
    
    for key, var in pairs(self.liveData) do
        local loc_path = tostring(key);
        --local filelist_content = cc.FileUtils:getInstance():getStringFromFile(loc_path);

        if loc_path ~= nil or loc_path ~= ""
        then
            local loc_md5 = PlatformMd5File(loc_path);
            loc_md5List[loc_path] = loc_md5;
        else
            return false;
        end
    end
    
    if loc_md5List ~=nil and next(loc_md5List) ~= nil then
        self.localData = loc_md5List;
        local loc_jsonList = json.encode(loc_md5List);
        self:saveToLocalFile(self.storagePath.."version.txt", self.localVersion);
        self:saveToLocalFile(self.storagePath.."filelist.txt", loc_jsonList);
        return true;
    else
        return false;
    end
end

--删除本地临时资源(在临时目录已经拷贝到正式目录后执行删除)
function FilesManager:disapperLocalTempData()
    cc.FileUtils:getInstance():removeDirectory(self.storageCachePath);
end

--删除本地正式资源(在发现新包的版本号大于本地更新目录版本号时执行删除)
function FilesManager:disapperLocalLiveData()
    cc.FileUtils:getInstance():removeDirectory(self.storagePath);
end

--释放资源
function FilesManager:disapper()
    self.liveVersion = nil;
    self.liveData = nil;
    self.localData = nil;
    self.diffData = nil;
end

return FilesManager;