require("hotfix.update.UpdateType");
local UpdateManager = class("UpdateManager");



function UpdateManager:ctor()
    self.status = nil;
end

function UpdateManager:init()

    local temp_file = require("hotfix.update.FilesManager");
    self.filesManager = temp_file.create(self);    
    
    self:refreshStatus(UpdateType.MAIN_STATUS.InitVersion);
    
    print("[UpdateManager] init()")
end

function UpdateManager.create(updateStatusFunc,loadProgress)
    local temp = UpdateManager.new();
    temp.onUpdateStatus = updateStatusFunc;
    temp.onLoadProgress = loadProgress;
    return temp;
end

function UpdateManager:refreshStatus(status)
    self.status = status;
    cclog("update status >>>"..self.status);
    if self.status == UpdateType.MAIN_STATUS.InitVersion then
        self.filesManager:getVersion();
        
    elseif self.status == UpdateType.MAIN_STATUS.FinishedVersion then
        local frameworkVersion = self.filesManager:getFrameworkVersion();
        
        if DataConfig.frameworkVersion < frameworkVersion then
            
            local function tempUpdateNo()
                self:refreshStatus(UpdateType.MAIN_STATUS.ReadyUpdate);
            end
            
            --向用户通知是否跳转下载页面
            self.onUpdateStatus("newFramework", tempUpdateNo);
        else
            self:refreshStatus(UpdateType.MAIN_STATUS.ReadyUpdate);
        end
        
    elseif self.status == UpdateType.MAIN_STATUS.ReadyUpdate then
        local liveVersion, localVersion = self.filesManager:getLiveAndLocalVersion();
        
        if liveVersion > localVersion then
            self:refreshStatus(UpdateType.MAIN_STATUS.InitFileList);
        else
            self:refreshStatus(UpdateType.MAIN_STATUS.NoUpdate);
        end
        
    elseif self.status == UpdateType.MAIN_STATUS.InitFileList then
        self.filesManager:getFileList();
        self.onUpdateStatus("checkFileList")--检查更新
    elseif self.status == UpdateType.MAIN_STATUS.FinishedFileList then
        self.filesManager:packageFilelistToMd5();
        local liveData, localData = self.filesManager:getLiveAndLocalFileList();
        local diffData, newLocalData, diffBytes = self:differentCheck(liveData, localData);
        self.diffData = diffData;
        self.newLocalData = newLocalData;
        
        if diffBytes > DataConfig.maxByte then
            local function tempUpdateYes()
                self:refreshStatus(UpdateType.MAIN_STATUS.UpdateFiles);
            end

            local function tempUpdateNo()
                self:refreshStatus(UpdateType.MAIN_STATUS.NoUpdate);
            end

            --向用户通知是否下载
            self.onUpdateStatus("downloadPopup", tempUpdateYes, tempUpdateNo,diffBytes);
        else 
            self:refreshStatus(UpdateType.MAIN_STATUS.UpdateFiles);
        end        
    elseif self.status == UpdateType.MAIN_STATUS.UpdateFiles then
        if  self.diffData == nil 
            or type(self.diffData) ~= "table"
            or next(self.diffData) == nil
            or self.diffData["all"] == nil
            or next(self.diffData["all"]) == nil
            or self.newLocalData == nil
            or type(self.newLocalData) ~= "table"
        then
            self:refreshStatus(UpdateType.MAIN_STATUS.ErrorLoadList);
        else
            self.onUpdateStatus("downloadStart");
            self.filesManager:updateDiffData(self.diffData,self.newLocalData);
        end
    
    elseif self.status == UpdateType.MAIN_STATUS.FinishedUpdateFiles then
        self.filesManager:saveLocalFiles();
        
    elseif self.status == UpdateType.MAIN_STATUS.Finished then
        self:disapper();
        
    elseif self.status == UpdateType.MAIN_STATUS.NoUpdate then
        self:disapper();
    elseif self.status == UpdateType.MAIN_STATUS.ErrorLoadList then
        self:disapper();
    end
end

function UpdateManager:loadProgress(loadedFile,maxFile)
    if self.onLoadProgress ~= nil then
        self.onLoadProgress(loadedFile,maxFile)
    end;
end;

function UpdateManager:differentCheck(liveData, localData)
    if liveData==nil then return nil, nil, 0; end

    localData = localData or {};

    local diffData = {all = {}};
    local newLocalData = {};
    local dataByte = 0;

    for key, li_md5 in pairs(liveData) do
        local diff_md5 = nil;
        local liveFileMd5 = li_md5[1];
        local liveFileByte = li_md5[2];
        --新增的文件
        if localData[key]==nil then
            diff_md5 = liveFileMd5;

        --判断MD5
        elseif localData[key]~=liveFileMd5 then 
            diff_md5 = liveFileMd5;
        end

        if diff_md5~=nil then
            local temp_file = {["md5"]=diff_md5};
            diffData["all"][key] = temp_file; --所有不同文件的列表
            newLocalData[key] = diff_md5;
            dataByte = dataByte + liveFileByte;
        else
            newLocalData[key] = localData[key];    
        end
    end

    for lo_key, lo_value in pairs(localData) do
        if newLocalData[lo_key]==nil then
            newLocalData[lo_key] = lo_value;
        end
    end

    return diffData, newLocalData, dataByte;
end

--加载或保存文件出错时处理方法
function UpdateManager:disapper()
    self.onUpdateStatus("complete");
    
    self.filesManager:disapperLocalTempData();
    self.filesManager:disapper();
    self.filesManager = nil;
end

return UpdateManager;